#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_postgresqlflexiblebackup
version_added: "3.6.0"
short_description: Manage PostgreSQL Flexible Backup instance
description:
    - Create or delete instance of PostgreSQL Flexible Backup.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource. You can obtain this value from the Azure Resource Manager API or the portal.
        required: True
        type: str
    server_name:
        description:
            - The name of the post gresql flexible server.
        required: True
        type: str
    backup_name:
        description:
            - The name of the backup.
        required: True
        type: str
    state:
        description:
            - Assert the state of the PostgreSQL Flexible backup. Use C(present) to create or update a backup and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - magodo (@magodo)
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Create (or update) PostgreSQL Flexible Backup
  azure_rm_postgresqlflexibledbackup:
    resource_group: myResourceGroup
    server_name: testserver
    backup_name: backup01

- name: Delete PostgreSQL Flexible Backup
  azure_rm_postgresqlflexibledbackup:
    resource_group: myResourceGroup
    server_name: testserver
    backup_name: backup01
    state: absent
'''

RETURN = '''
backup:
    description:
        - A dictionaries containing facts for PostgreSQL Flexible Backup.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Fully qualified resource ID for the resource.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.DBforPostgreSQL/flexibleServers/posttest/backups/fredbackup"
        backup_name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: fredbackup
        server_name:
            description:
                - The post gresql flexible server name.
            returned: always
            type: str
            sample: posttest
        resource_group:
            description:
                - Name of the resource group.
            returned: always
            type: str
            sample: testRG
        type:
            description:
                - The type of the resource.
            returned: always
            type: str
            sample: Microsoft.DBforPostgreSQL/flexibleServers/backups
        completed_time:
            description:
                - Backup completed time (ISO8601 format).
            type: str
            returned: always
            sample: "2025-04-17T08:11:58.756273+00:00"
        backup_type:
            description:
                - Backup type.
            type: str
            returned: always
            sample: Full
        source:
            description:
                - Backup source.
            type: str
            returned: always
            sample: Automatic
'''


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMPostgreSqlFlexibleBackup(AzureRMModuleBase):
    """Configuration class for an Azure RM PostgreSQL Flexible Backup resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            server_name=dict(
                type='str',
                required=True
            ),
            backup_name=dict(
                type='str',
                required=True
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.server_name = None
        self.backup_name = None

        self.results = dict(changed=False)
        self.state = None

        super(AzureRMPostgreSqlFlexibleBackup, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                              supports_check_mode=True,
                                                              supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

        old_response = None
        response = None
        changed = False

        old_response = self.get_postgresqlflexiblebackup()

        if not old_response:
            self.log("PostgreSQL Flexible Backup instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                changed = True
                if not self.check_mode:
                    response = self.create_postgresqlflexiblebackup()
        else:
            self.log("PostgreSQL Flexible Backup instance already exists")
            if self.state == 'absent':
                changed = True
                if not self.check_mode:
                    response = self.delete_postgresqlflexiblebackup()
            else:
                if self.check_mode:
                    changed = True
                response = old_response

        self.results['backup'] = response
        self.results['changed'] = changed
        return self.results

    def create_postgresqlflexiblebackup(self):
        '''
        Creates PostgreSQL Flexible Backup with the specified configuration.

        :return: deserialized PostgreSQL Flexible Backup instance state dictionary
        '''
        self.log("Creating the PostgreSQL Flexible Backup instance {0}".format(self.backup_name))

        try:
            response = self.postgresql_flexible_client.backups_automatic_and_on_demand.begin_create(resource_group_name=self.resource_group,
                                                                                                    server_name=self.server_name,
                                                                                                    backup_name=self.backup_name)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the PostgreSQL Flexible Backup instance.')
            self.fail("Error creating the PostgreSQL Flexible Backup instance: {0}".format(str(exc)))
        return self.format_item(response)

    def delete_postgresqlflexiblebackup(self):
        '''
        Deletes specified PostgreSQL Flexible Backup instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the PostgreSQL Flexible Backup instance {0}".format(self.backup_name))
        try:
            self.postgresql_flexible_client.backups_automatic_and_on_demand.begin_delete(resource_group_name=self.resource_group,
                                                                                         server_name=self.server_name,
                                                                                         backup_name=self.backup_name)
        except Exception as ec:
            self.log('Error attempting to delete the PostgreSQL Flexible Backup instance.')
            self.fail("Error deleting the PostgreSQL Flexible Backup instance: {0}".format(str(ec)))

    def get_postgresqlflexiblebackup(self):
        '''
        Gets the properties of the specified PostgreSQL Flexible Backup.

        :return: deserialized PostgreSQL Flexible Backup instance state dictionary
        '''
        self.log("Checking if the PostgreSQL Flexible Backup instance {0} is present".format(self.backup_name))
        found = False
        try:
            response = self.postgresql_flexible_client.backups_automatic_and_on_demand.get(resource_group_name=self.resource_group,
                                                                                           server_name=self.server_name,
                                                                                           backup_name=self.backup_name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("PostgreSQL Flexible Backup instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the PostgreSQL Flexible Backup instance. Exception as {0}'.format(e))
        if found is True:
            return self.format_item(response)

        return None

    def format_item(self, item):
        return dict(
            resource_group=self.resource_group,
            server_name=self.server_name,
            backup_type=item.backup_type,
            completed_time=item.completed_time,
            id=item.id,
            name=item.name,
            source=item.source,
            type=item.type
        )


def main():
    """Main execution"""
    AzureRMPostgreSqlFlexibleBackup()


if __name__ == '__main__':
    main()



# Copyright (c) Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # Standard documentation fragment
    DOCUMENTATION = r"""
options:
  api_token:
    description:
      - Online OAuth token.
    type: str
    required: true
    aliases: [oauth_token]
  api_url:
    description:
      - Online API URL.
    type: str
    default: 'https://api.online.net'
    aliases: [base_url]
  api_timeout:
    description:
      - HTTP timeout to Online API in seconds.
    type: int
    default: 30
    aliases: [timeout]
  validate_certs:
    description:
      - Validate SSL certs of the Online API.
    type: bool
    default: true
notes:
  - Also see the API documentation on U(https://console.online.net/en/api/).
  - If O(api_token) is not set within the module, the following environment variables can be used in decreasing order of precedence
    E(ONLINE_TOKEN), E(ONLINE_API_KEY), E(ONLINE_OAUTH_TOKEN), E(ONLINE_API_TOKEN).
  - If one wants to use a different O(api_url) one can also set the E(ONLINE_API_URL) environment variable.
"""



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import json
import os
import select
import time
import typing
from pty import openpty
from subprocess import PIPE
from subprocess import Popen

if typing.TYPE_CHECKING:
    from typing import Any
    from typing import Callable
    from typing import Dict
    from typing import Iterator
    from typing import NoReturn
    from typing import Optional

    verbosity_display_type = Callable[[int, str], None]

from ansible.errors import AnsibleConnectionFailure
from ansible.module_utils.common.text.converters import to_text

from ansible_collections.amazon.aws.plugins.plugin_utils.text import filter_ansi


class SSMProcessManagerTimeOutFailure(AnsibleConnectionFailure):
    pass


def _create_polling_obj(fd: Any) -> Any:
    """create polling object using select.poll, this is helpful for unit testing"""
    poller = select.poll()
    poller.register(fd, select.POLLIN)
    return poller


class ProcessManager:
    def __init__(
        self, instance_id: str, session: Any, stdout: Any, timeout: int, verbosity_display: verbosity_display_type
    ) -> None:
        """
        Initialise the ProcessManager.

        :param instance_id: The EC2 instance ID
        :param session: The subprocess session
        :param stdout: The stdout file descriptor
        :param timeout: Connection timeout in seconds
        :param verbosity_display: Function for logging verbosity messages
        """
        self._session = session
        self._stdout = stdout
        self.verbosity_display = verbosity_display
        self._timeout = timeout
        self._poller = None
        self.instance_id = instance_id
        self._session_id = None
        self._poller = _create_polling_obj(self._stdout)
        self._has_timeout = False
        self._stderr_accumulator = ""

    def stdin_write(self, command: bytes | str) -> None:
        if isinstance(command, str):
            command = command.encode("utf-8")
        self.verbosity_display(5, f"stdin_write: Writing {len(command)} bytes")
        self.verbosity_display(5, f"stdin_write: First 200 bytes: {repr(command[:200])}")
        self._session.stdin.write(command)
        self._session.stdin.flush()
        self.verbosity_display(5, "stdin_write: Flushed")

    def stdout_read_text(self, size: int = 1024) -> str:
        return to_text(self._stdout.read(size), encoding="utf-8", errors="surrogate_or_strict")

    def stdout_readline(self) -> str:
        self.verbosity_display(5, "stdout_readline: About to call readline()")
        result = self._stdout.readline()
        self.verbosity_display(5, f"stdout_readline: readline() returned {len(result)} bytes")
        # DEBUG: Level 6 for long-term debugging of encoding issues
        self.verbosity_display(6, f"stdout_readline: Raw bytes: {result[:200]}")
        # Decode as UTF-8 with surrogate escapes to handle any encoding issues
        decoded = to_text(result, encoding="utf-8", errors="surrogate_or_strict")
        self.verbosity_display(6, f"stdout_readline: Decoded string: {repr(decoded[:200])}")
        return decoded

    def flush_stderr(self) -> str:
        """Read available stderr, accumulate it, and return all accumulated stderr.

        This method reads any currently available stderr data without blocking,
        accumulates it internally, and returns the full accumulated stderr.
        The accumulator is then cleared so subsequent calls start fresh.

        :returns: All accumulated stderr since last flush
        """

        poller = _create_polling_obj(self._session.stderr)
        self.verbosity_display(5, "FLUSH_STDERR: Checking session.poll() status")
        poll_result = self._session.poll()
        self.verbosity_display(5, f"FLUSH_STDERR: session.poll() returned {poll_result}")
        while poll_result is None:
            self.verbosity_display(5, "FLUSH_STDERR: About to poll stderr with 1ms timeout")
            poll_res = poller.poll(1)
            self.verbosity_display(5, f"FLUSH_STDERR: stderr poll returned {poll_res}")
            if not poll_res:
                break
            self.verbosity_display(5, "FLUSH_STDERR: About to read stderr line")
            line = to_text(self._session.stderr.readline(), encoding="utf-8", errors="surrogate_or_strict")
            self.verbosity_display(5, f"FLUSH_STDERR: stderr readline returned {len(line)} chars")
            self.verbosity_display(4, f"stderr line: {repr(line)}")
            self._stderr_accumulator = self._stderr_accumulator + line
            self.verbosity_display(5, "FLUSH_STDERR: Checking session.poll() status")
            poll_result = self._session.poll()
            self.verbosity_display(5, f"FLUSH_STDERR: session.poll() returned {poll_result}")

        # Return accumulated stderr and clear the accumulator
        result = self._stderr_accumulator
        self._stderr_accumulator = ""
        return result

    def poll_stdout(self, length: int = 1000) -> bool:
        self.verbosity_display(5, f"poll_stdout: About to poll with {length}ms timeout")
        result = bool(self._poller.poll(length))
        self.verbosity_display(5, f"poll_stdout({length}ms) -> {result}")
        return result

    def poll(self, label: str, cmd: str) -> Iterator[bool]:
        """Poll session to retrieve content from stdout.

        :param label: A label for the display (EXEC, PRE...)
        :param cmd: The command being executed
        """
        start = round(time.time())
        yield self.poll_stdout()
        self.verbosity_display(5, f"{label} POLL: Checking session.poll() status")
        poll_result = self._session.poll()
        self.verbosity_display(5, f"{label} POLL: session.poll() returned {poll_result}")
        while poll_result is None:
            remaining = start + self._timeout - round(time.time())
            self.verbosity_display(4, f"{label} remaining: {remaining} second(s)")
            if remaining < 0:
                self._has_timeout = True
                raise SSMProcessManagerTimeOutFailure(f"{label} command '{cmd}' timeout on host: {self.instance_id}")
            yield self.poll_stdout()
            self.verbosity_display(5, f"{label} POLL: Checking session.poll() status")
            poll_result = self._session.poll()
            self.verbosity_display(5, f"{label} POLL: session.poll() returned {poll_result}")

    def _try_match(
        self, stdout: str, match: str | Callable[[str], bool], is_windows: bool, label: str, elapsed: float
    ) -> bool:
        """Check if match is found in stdout and log result.

        For string matches, searches directly. For callable matches, tries matching
        on both raw and ANSI-filtered stdout.

        :param stdout: Accumulated stdout content.
        :param match: String to search for or callable that returns True when matched.
        :param is_windows: Whether output is from Windows (for ANSI filtering).
        :param label: Label for logging.
        :param elapsed: Time elapsed since wait started.
        :return: True if match found, False otherwise.
        """
        if isinstance(match, str):
            if stdout.find(match) != -1:
                self.verbosity_display(
                    4, f"{label} WAIT_FOR_MATCH: MATCHED! Found string in stdout after {elapsed:.2f}s"
                )
                return True
            self.verbosity_display(5, f"{label} WAIT_FOR_MATCH: No match yet, last 100 chars: {repr(stdout[-100:])}")
            return False

        # Callable match - try both raw and filtered
        stdout_filtered = filter_ansi(stdout, is_windows)
        if match(stdout) or match(stdout_filtered):
            self.verbosity_display(4, f"{label} WAIT_FOR_MATCH: MATCHED! Pattern matched stdout after {elapsed:.2f}s")
            return True

        self.verbosity_display(5, f"{label} WAIT_FOR_MATCH: No pattern match yet")
        self.verbosity_display(5, f"{label} WAIT_FOR_MATCH: Last 100 chars (raw): {repr(stdout[-100:])}")
        self.verbosity_display(5, f"{label} WAIT_FOR_MATCH: Last 100 chars (filtered): {repr(stdout_filtered[-100:])}")
        return False

    def _log_progress_if_needed(
        self,
        current_time: float,
        last_progress_log: float,
        wait_start: float,
        label: str,
        line_count: int,
        poll_count: int,
        stdout_len: int,
    ) -> float:
        """Log progress message if enough time has elapsed.

        :param current_time: Current timestamp.
        :param last_progress_log: Timestamp of last progress log.
        :param wait_start: Timestamp when waiting started.
        :param label: Label for logging.
        :param line_count: Number of lines read so far.
        :param poll_count: Number of polls performed.
        :param stdout_len: Length of accumulated stdout.
        :return: Updated last_progress_log timestamp.
        """
        if current_time - last_progress_log >= 5.0:
            elapsed = current_time - wait_start
            self.verbosity_display(
                3,
                f"{label} WAIT_FOR_MATCH: Still waiting after {elapsed:.1f}s ({line_count} lines, {poll_count} polls, {stdout_len} bytes collected)",
            )
            return current_time
        return last_progress_log

    def _check_and_log_stderr(self, label: str) -> None:
        """Check stderr and log any output found.

        :param label: Label for logging.
        """
        stderr_output = self.flush_stderr()
        if stderr_output:
            self.verbosity_display(4, f"{label} WAIT_FOR_MATCH: stderr while waiting: {repr(stderr_output)}")

    def _log_match_start(self, label: str, cmd: str | bytes, match: str | Callable[[str], bool]) -> None:
        """Log the start of waiting for a match.

        :param label: Label for logging.
        :param cmd: Command being executed.
        :param match: Pattern to match (string or callable).
        """
        self.verbosity_display(4, f"{label} WAIT_FOR_MATCH: Starting to wait for pattern")
        if isinstance(match, str):
            self.verbosity_display(4, f"{label} WAIT_FOR_MATCH: Looking for string: {repr(match)}")
        else:
            self.verbosity_display(4, f"{label} WAIT_FOR_MATCH: Looking for pattern match")
        self.verbosity_display(4, f"{label} WAIT_FOR_MATCH: Command = {to_text(cmd)}")

    def _check_and_warn_command_echo(self, stdout: str, line_count: int, is_windows: bool, label: str) -> None:
        """Check for PowerShell command echo and log warning if detected.

        :param stdout: Accumulated stdout content.
        :param line_count: Number of lines read.
        :param is_windows: Whether output is from Windows.
        :param label: Label for logging.
        """
        if not (is_windows and label == "EXEC_COMMUNICATE" and line_count > 5):
            return

        # Look for signs of PowerShell command echo (base64 or PowerShell keywords)
        if "powershell" in stdout.lower() or any(
            char in stdout for char in ["A", "B", "C", "D"] if stdout.count(char) > 50
        ):
            self.verbosity_display(
                2,
                f"{label} WAIT_FOR_MATCH: WARNING - Detected possible command echo in {line_count} lines before marker. "
                "PSReadLine may not have been removed properly.",
            )

    def wait_for_match(
        self, label: str, cmd: str | bytes, match: str | Callable[[str], bool], is_windows: bool = False
    ) -> None:
        """Wait for stdout to match a pattern.

        For callable matches (e.g. regex), tries matching on both raw and ANSI-filtered
        stdout to handle cases where terminal control codes might interfere with matching.

        Uses readline() to avoid consuming additional output beyond the matched line,
        preventing race conditions where both start and end markers are read together.

        :param label: Label for logging purposes.
        :param cmd: The command being executed.
        :param match: Either a string to search for or a callable that returns True when matched.
        :param is_windows: Whether the output is from a Windows host (for ANSI filtering).
        """
        stdout = ""
        wait_start = time.time()
        last_progress_log = wait_start
        poll_count = 0
        line_count = 0

        self._log_match_start(label, cmd, match)
        self._check_and_log_stderr(label)

        for result in self.poll(label=label, cmd=to_text(cmd)):
            poll_count += 1
            current_time = time.time()

            last_progress_log = self._log_progress_if_needed(
                current_time, last_progress_log, wait_start, label, line_count, poll_count, len(stdout)
            )

            if result:
                line = to_text(self.stdout_readline())
                line_count += 1
                self.verbosity_display(4, f"{label} WAIT_FOR_MATCH: stdout line {line_count}: {repr(line)}")
                stdout += line

                elapsed = time.time() - wait_start
                if self._try_match(stdout, match, is_windows, label, elapsed):
                    break
            else:
                self._check_and_log_stderr(label)

        total_duration = time.time() - wait_start
        self.verbosity_display(
            4,
            f"{label} WAIT_FOR_MATCH: Complete after {total_duration:.2f}s - {line_count} lines, {poll_count} polls, {len(stdout)} bytes collected",
        )
        self.verbosity_display(5, f"{label} WAIT_FOR_MATCH: Full stdout collected: {repr(stdout)}")

        self._check_and_warn_command_echo(stdout, line_count, is_windows, label)

    def terminate(self) -> None:
        if self._has_timeout:
            self._session.terminate()
        else:
            cmd = b"\nexit\n"
            self._session.communicate(cmd)


class SSMSessionManager:
    def __init__(
        self, ssm_client: Any, instance_id: str, ssm_timeout: int, verbosity_display: verbosity_display_type
    ) -> None:
        """
        Initialise the SSMSessionManager.

        :param ssm_client: The boto3 SSM client
        :param instance_id: The EC2 instance ID
        :param ssm_timeout: Connection timeout in seconds
        :param verbosity_display: Function for logging verbosity messages
        """
        self._session_id = None
        self._instance_id = instance_id
        self.verbosity_display = verbosity_display
        self._client = ssm_client
        self._process_mgr = None
        self._timeout = ssm_timeout
        self._session = None

    @property
    def instance_id(self) -> str:
        return self._instance_id

    @property
    def session(self) -> Any:
        return self._session

    @property
    def stdout(self) -> Any:
        return self._stdout

    def __getattr__(self, attr: str) -> Callable:
        if self._process_mgr and hasattr(self._process_mgr, attr):
            return getattr(self._process_mgr, attr)
        raise AttributeError(f"class SSMSessionManager has no attribute '{attr}'")

    def start_session(
        self,
        executable: str,
        document_name: Optional[str],
        region_name: Optional[str],
        profile_name: str,
        parameters: Dict[str, Any] = None,
    ) -> None:
        """Start SSM Session manager session and eventually prepare terminal"""

        self.verbosity_display(2, f"ESTABLISH SSM CONNECTION TO: {self.instance_id}")
        if parameters is None:
            parameters = {}
        start_session_args = dict(Target=self.instance_id, Parameters=parameters)
        if document_name is not None:
            start_session_args["DocumentName"] = document_name
        response = self._client.start_session(**start_session_args)
        self._session_id = response["SessionId"]
        self.verbosity_display(3, f"SSM CONNECTION ID: {self._session_id}")

        cmd = [
            executable,
            json.dumps(response),
            region_name,
            "StartSession",
            profile_name,
            json.dumps({"Target": self.instance_id}),
            self._client.meta.endpoint_url,
        ]

        self.verbosity_display(4, f"SSM COMMAND: {to_text(cmd)}")

        stdout_r, stdout_w = openpty()
        self._session = Popen(cmd, stdin=PIPE, stdout=stdout_w, stderr=PIPE, close_fds=True, bufsize=0)

        os.close(stdout_w)
        stdout = os.fdopen(stdout_r, "rb", 0)

        self._process_mgr = ProcessManager(
            instance_id=self.instance_id,
            session=self._session,
            stdout=stdout,
            timeout=self._timeout,
            verbosity_display=self.verbosity_display,
        )

    def terminate(self) -> None:
        if self._process_mgr:
            self._process_mgr.terminate()
        if self._session_id and self._client:
            try:
                self._client.terminate_session(SessionId=self._session_id)
            except ReferenceError:
                # Client may have been garbage collected during cleanup
                pass
            self._session_id = None



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Shreyas Srish (@shrsr) <ssrish@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_interface_policy_hsrp
short_description: Manage HSRP interface policies (hsrp:IfPol)
description:
- Manage HSRP interface policies on Cisco ACI fabrics.
options:
  tenant:
    description:
    - The name of the Tenant the HSRP interface policy should belong to.
    type: str
    aliases: [ tenant_name ]
  hsrp:
    description:
    - The HSRP interface policy name.
    type: str
    aliases: [ hsrp_interface, name ]
  description:
    description:
    - The description of the HSRP interface.
    type: str
    aliases: [ descr ]
  controls:
    description:
    - The interface policy controls.
    type: list
    elements: str
    choices: [ bfd, bia ]
  delay:
    description:
    - The administrative port delay of HSRP interface policy.
    - This is only valid in range between 1 and 10000.
    type: int
  reload_delay:
    description:
    - The option for reload delay of HSRP interface policy.
    - This is only valid in range between 1 and 10000.
    type: int
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(tenant) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(hsrp:IfPol).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Shreyas Srish (@shrsr)
"""

EXAMPLES = r"""
- name: Add a HSRP interface policy
  cisco.aci.aci_interface_policy_hsrp:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    hsrp: hsrp1
    controls: bfd
    delay: 50
    reload_delay: 100
    state: present
  delegate_to: localhost

- name: Delete a HSRP interface policy
  cisco.aci.aci_interface_policy_hsrp:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    hsrp: hsrp1
    state: absent
  delegate_to: localhost

- name: Query a HSRP interface policy
  cisco.aci.aci_interface_policy_hsrp:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    hsrp: hsrp1
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all HSRP interface policies in tenant production
  cisco.aci.aci_interface_policy_hsrp:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        hsrp=dict(type="str", aliases=["hsrp_interface", "name"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        controls=dict(type="list", elements="str", choices=["bfd", "bia"]),
        reload_delay=dict(type="int"),
        delay=dict(type="int"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "hsrp"]],
            ["state", "present", ["tenant", "hsrp"]],
        ],
    )

    aci = ACIModule(module)

    tenant = module.params.get("tenant")
    hsrp = module.params.get("hsrp")
    description = module.params.get("description")
    controls = ",".join(module.params.get("controls")) if module.params.get("controls") else None

    reload_delay = module.params.get("reload_delay")
    if reload_delay is not None and reload_delay not in range(1, 10000):
        module.fail_json(msg="Parameter 'reload_delay' is only valid in range between 1 and 10000.")

    delay = module.params.get("delay")
    if delay is not None and delay not in range(1, 10000):
        module.fail_json(msg="Parameter 'delay' is only valid in range between 1 and 10000.")

    state = module.params.get("state")

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="hsrpIfPol",
            aci_rn="hsrpIfPol-{0}".format(hsrp),
            module_object=hsrp,
            target_filter={"name": hsrp},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="hsrpIfPol",
            class_config=dict(
                name=hsrp,
                descr=description,
                ctrl=controls,
                reloadDelay=reload_delay,
                delay=delay,
            ),
        )

        aci.get_diff(aci_class="hsrpIfPol")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_packet_capture_captures
short_description: Resource module for organizations _devices _packet _capture _captures
description:
  - Manage operations create and delete of the resource organizations _devices _packet
    _capture _captures. - > Perform a packet capture on a device and store in Meraki
    Cloud. Only a single switch may be chosen per request, while multiple access points
    are allowed at once.
  - Delete a single packet capture from cloud using captureId.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  advanced:
    description: Advanced filters for IOSXE devices (supported for Campus Gateway
      devices only).
    suboptions:
      bufferFiles:
        description: Number of buffer files for circular buffer capture (1-5 for Campus
          Gateway devices).
        type: int
      captureType:
        description: Type of capture. Possible values linear (default), circular.
        type: str
      controlPlaneDirection:
        description: Direction for control plane capture.
        type: str
      innerFilterMacs:
        description: Inner MAC address filter for tunneled traffic (up to 5 MAC addresses
          for Campus Gateway devices).
        elements: str
        type: list
      maxFilesize:
        description: Maximum file size in megabytes (MB). Range 1-100 MB when bufferFiles=1,
          1-500 MB when bufferFiles=2-5.
        type: int
      packetsPerSecond:
        description: Packets per second limit for Campus Gateway devices (1-1000000).
        type: int
      physicalInterfaceDirection:
        description: Direction for physical interface capture.
        type: str
    type: dict
  captureId:
    description: CaptureId path parameter. Capture ID.
    type: str
  destination:
    description: Destination of packet capture file. Possible values upload_to_cloud.
    type: str
  duration:
    description: Duration in seconds of packet capture.
    type: int
  filterExpression:
    description: Filter expression for packet capture.
    type: str
  interface:
    description: Interface of the device.
    type: str
  name:
    description: Name of packet capture file.
    type: str
  notes:
    description: Reason for taking the packet capture.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  outputType:
    description: Output type of packet capture file. Possible values text, pcap, cloudshark,
      or upload_to_cloud.
    type: str
  ports:
    description: Ports of packet capture file, comma-separated.
    type: str
  serials:
    description: The serial(s) of the device(s).
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations createOrganizationDevicesPacketCaptureCapture
    description: Complete reference of the createOrganizationDevicesPacketCaptureCapture
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-devices-packet-capture-capture
  - name: Cisco Meraki documentation for organizations deleteOrganizationDevicesPacketCaptureCapture
    description: Complete reference of the deleteOrganizationDevicesPacketCaptureCapture
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-devices-packet-capture-capture
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_devices_packet_capture_capture,
    organizations.Organizations.delete_organization_devices_packet_capture_capture,
  - Paths used are
    post /organizations/{organizationId}/devices/packetCapture/captures,
    delete /organizations/{organizationId}/devices/packetCapture/captures/{captureId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_devices_packet_capture_captures:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    advanced:
      bufferFiles: 5
      captureType: circular
      controlPlaneDirection: both
      innerFilterMacs:
        - aa:bb:cc:dd:ee:ff
        - 11:22:33:44:55:66
      maxFilesize: 10
      packetsPerSecond: 10000
      physicalInterfaceDirection: both
    destination: upload_to_cloud
    duration: 3
    filterExpression: host 10.1.27.253
    interface: wireless
    name: Capture no. 3
    notes: Debugging connectivity issue...
    organizationId: string
    outputType: upload_to_cloud
    ports: 1, 3
    serials:
      - Q234-ABCD-5678
- name: Delete by id
  cisco.meraki.organizations_devices_packet_capture_captures:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    captureId: string
    organizationId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "captureId": "string",
      "network": {
        "id": "string",
        "name": "string"
      },
      "devices": [
        {}
      ],
      "device": {
        "name": "string",
        "serial": "string"
      },
      "admin": {
        "id": "string",
        "name": "string"
      },
      "client": {
        "id": "string",
        "mac": "string"
      },
      "details": [
        {
          "name": "string",
          "value": "string",
          "productType": "string"
        }
      ],
      "name": "string",
      "startTs": "string",
      "ports": "string",
      "status": "string",
      "errorMessage": "string",
      "destination": "string",
      "process": "string",
      "file": {
        "size": 0
      },
      "duration": 0,
      "filterExpression": "string",
      "counts": {
        "packets": {
          "total": 0
        }
      },
      "interface": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_smtp_policy_info
short_description: Gather information about SMTP Policies in Cisco Intersight
description:
  - Gather information about SMTP Policies in L(Cisco Intersight,https://intersight.com).
  - Information can be filtered by O(organization) and O(name).
  - If no filters are passed, all SMTP Policies will be returned.
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the organization the SMTP Policy belongs to.
    type: str
  name:
    description:
      - The name of the SMTP Policy to gather information from.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Fetch a specific SMTP Policy by name
  cisco.intersight.intersight_smtp_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "smtp-policy-test"

- name: Fetch all SMTP Policies in a specific Organization
  cisco.intersight.intersight_smtp_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"

- name: Fetch all SMTP Policies
  cisco.intersight.intersight_smtp_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": [
      {
        "Name": "smtp-policy-test",
        "ObjectType": "smtp.Policy",
        "Description": "SMTP Policy for Alerting",
        "Enabled": true,
        "SmtpServer": "smtp.example.com",
        "SmtpPort": 25,
        "MinSeverity": "warning",
        "SenderEmail": "alerts@example.com",
        "SmtpRecipients": [
            "admin@example.com",
            "devops@example.com"
        ],
        "Tags": []
      }
    ]
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str'),
        name=dict(type='str')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    intersight = IntersightModule(module)

    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/smtp/Policies'
    query_params = intersight.set_query_params()

    intersight.get_resource(
        resource_path=resource_path,
        query_params=query_params,
        return_list=True
    )

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2017 Zim Kalinowski, <zikalino@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_keyvault
version_added: "0.1.2"
short_description: Manage Key Vault instance
description:
    - Create, update and delete instance of Key Vault.

options:
    resource_group:
        description:
            - The name of the Resource Group to which the server belongs.
        required: True
        type: str
    vault_name:
        description:
            - Name of the vault.
            - It is mutually exclusive with I(hsm_name)
        required: False
        type: str
    hsm_name:
        description:
            - Name of the HSM.
            - It is mutually exclusive with I(vault_name)
        required: False
        type: str
    administrators:
        description:
            - List of administrator OID's for data plane operations for Managed HSM.
            - It is mutually exclusive with I(vault_name)
        required: False
        type: list
        elements: str
        default: []
    identity:
        description:
            - Identity for the HSM, not valid for vault_name.
            - It is mutually exclusive with I(vault_name)
        type: dict
        version_added: '3.0.0'
        suboptions:
            type:
                description:
                    - Type of the managed identity
                choices:
                    - UserAssigned
                    - None
                default: None
                type: str
            user_assigned_identity:
                description:
                    - User Assigned Managed Identity associated to this resource
                required: false
                type: str
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
        type: str
    vault_tenant:
        description:
            - The Azure Active Directory tenant ID that should be used for authenticating requests to the key vault.
        type: str
        aliases:
          - tenant_id
    sku:
        description:
            - SKU details.
        type: dict
        suboptions:
            family:
                description:
                    - SKU family name.
                type: str
                required: True
            name:
                description:
                    - SKU name to specify whether the key vault is a standard vault or a premium vault.
                required: True
                type: str
                choices:
                    - 'standard'
                    - 'premium'
    access_policies:
        description:
            - An array of 0 to 16 identities that have access to the key vault.
            - All identities in the array must use the same tenant ID as the key vault's tenant ID.
            - It is mutually exclusive with I(hsm_name)
        type: list
        elements: dict
        suboptions:
            tenant_id:
                description:
                    - The Azure Active Directory tenant ID that should be used for authenticating requests to the key vault.
                    - Current keyvault C(tenant_id) value will be used if not specified.
                type: str
            object_id:
                description:
                    - The object ID of a user, service principal or security group in the Azure Active Directory tenant for the vault.
                    - The object ID must be unique for the list of access policies.
                    - Please note this is not application id. Object id can be obtained by running "az ad sp show --id <application id>".
                type: str
                required: True
            application_id:
                description:
                    -  Application ID of the client making request on behalf of a principal.
                type: str
            keys:
                description:
                    - List of permissions to keys.
                type: list
                elements: str
                choices:
                    - 'encrypt'
                    - 'decrypt'
                    - 'wrapkey'
                    - 'unwrapkey'
                    - 'sign'
                    - 'verify'
                    - 'get'
                    - 'list'
                    - 'create'
                    - 'update'
                    - 'import'
                    - 'delete'
                    - 'backup'
                    - 'restore'
                    - 'recover'
                    - 'purge'
            secrets:
                description:
                    - List of permissions to secrets.
                type: list
                elements: str
                choices:
                    - 'get'
                    - 'list'
                    - 'set'
                    - 'delete'
                    - 'backup'
                    - 'restore'
                    - 'recover'
                    - 'purge'
            certificates:
                description:
                    - List of permissions to certificates.
                type: list
                elements: str
                choices:
                    - 'get'
                    - 'list'
                    - 'delete'
                    - 'create'
                    - 'import'
                    - 'update'
                    - 'managecontacts'
                    - 'getissuers'
                    - 'listissuers'
                    - 'setissuers'
                    - 'deleteissuers'
                    - 'manageissuers'
                    - 'recover'
                    - 'purge'
                    - 'backup'
                    - 'restore'
            storage:
                description:
                    - List of permissions to storage accounts.
                type: list
                elements: str
    enabled_for_deployment:
        description:
            - Property to specify whether Azure Virtual Machines are permitted to retrieve certificates stored as secrets from the key vault.
            - It is mutually exclusive with I(hsm_name)
        type: bool
    enabled_for_disk_encryption:
        description:
            - Property to specify whether Azure Disk Encryption is permitted to retrieve secrets from the vault and unwrap keys.
            - It is mutually exclusive with I(hsm_name)
        type: bool
    enabled_for_template_deployment:
        description:
            - Property to specify whether Azure Resource Manager is permitted to retrieve secrets from the key vault.
            - It is mutually exclusive with I(hsm_name)
        type: bool
    enable_soft_delete:
        description:
            - Property to specify whether the soft delete functionality is enabled for this key vault.
        type: bool
        default: True
    enable_purge_protection:
        description:
            - Property specifying whether protection against purge is enabled for this vault.
        type: bool
        default: False
    enable_rbac_authorization:
        description:
            - Property that controls how data actions are authorized.
            - When I(enable_rbac_authorization=true), the key vault will use Role Based Access Control (RBAC) for authorization of data actions,
              and the access policies specified in vault properties will be  ignored.
            - When I(enable_rbac_authorization=false), the key vault will use the access policies specified in vault properties,
              and any policy stored on Azure Resource Manager will be ignored.
            - If null or not specified, the value of this property will not change.
            - Starting with API version 2026-02-01, Azure defaults this to true (RBAC) if not specified.
        type: bool
    soft_delete_retention_in_days:
        description:
            - Property specifying the number of days to retain deleted vaults.
        type: int
    recover_mode:
        description:
            - Create vault in recovery mode.
        type: bool
    public_network_access:
        description:
            - Property to specify whether the vault will accept traffic from public internet.
        type: str
        choices:
            - Disabled
            - Enabled
    network_acls:
        description:
            - A collection of rules governing the accessibility of the vault from specific network locations.
        type: dict
        suboptions:
            bypass:
                description:
                    - Tells what traffic can bypass network rules.
                    - If not specified the default is 'AzureServices'.
                type: str
                choices:
                    - AzureServices
                    - None
                default: AzureServices
            default_action:
                description:
                    - The default action when no rule from ipRules and from virtualNetworkRules match.
                    - This is only used after the bypass property has been evaluated.
                type: str
                choices:
                    - Allow
                    - Deny
            ip_rules:
                description:
                    - The list of IP address rules.
                type: list
                elements: dict
                suboptions:
                    value:
                        description:
                            - An IPv4 address range in CIDR notation.
                            - Such as C(124.56.78.91) (simple IP address) or C(124.56.78.0/24) (all addresses that start with 124.56.78).
                        type: str
            virtual_network_rules:
                description:
                    - The list of virtual network rules.
                type: list
                elements: dict
                suboptions:
                    id:
                        description:
                            - Full resource id of a vnet subnet.
                        type: str
                    ignore_missing_vnet_service_endpoint:
                        description:
                            - Property to specify whether NRP will ignore the check if parent subnet has serviceEndpoints configured.
                        type: bool
    is_purge_deleted:
        description:
            - Whether permanently deletes the specified vault. aka Purges the deleted Azure key vault
            - Default value is C(True)
            - When I(is_purge_deleted) is specified, the I(location) has to be configured.
              If not configured, the default locatioin of the resource group will be used.
        type: bool
        default: False
    state:
        description:
            - Assert the state of the KeyVault. Use C(present) to create or update an KeyVault and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Create instance of Key Vault
  azure_rm_keyvault:
    resource_group: myResourceGroup
    vault_name: samplekeyvault
    enabled_for_deployment: true
    enable_rbac_authorization: true
    vault_tenant: 72f98888-8666-4144-9199-2d7cd0111111
    sku:
      name: standard
      family: A
    access_policies:
      - tenant_id: 72f98888-8666-4144-9199-2d7cd0111111
        object_id: 99998888-8666-4144-9199-2d7cd0111111
        keys:
          - get
          - list
'''

RETURN = '''
id:
    description:
        - The Azure Resource Manager resource ID for the key vault.
    returned: always
    type: str
    sample: id
'''

import time
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.polling import LROPoller
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.keyvault import KeyVaultManagementClient
    from azure.mgmt.keyvault.models import (ManagedServiceIdentity, UserAssignedIdentity, NetworkRuleSet,
                                            NetworkRuleBypassOptions, NetworkRuleAction, ManagedHsmProperties,
                                            ManagedHsm, ManagedHsmSku)
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMVaults(AzureRMModuleBaseExt):
    """Configuration class for an Azure RM Key Vault resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            vault_name=dict(
                type='str'
            ),
            hsm_name=dict(
                type='str'
            ),
            location=dict(
                type='str'
            ),
            vault_tenant=dict(
                type='str',
                aliases=['tenant_id']
            ),
            sku=dict(
                type='dict'
            ),
            administrators=dict(
                type='list',
                default=[],
                elements='str'
            ),
            identity=dict(
                type='dict',
                options=dict(
                    type=dict(
                        type='str',
                        choices=['UserAssigned', 'None'],
                        default='None'
                    ),
                    user_assigned_identity=dict(
                        type="str",
                    ),
                ),
            ),
            access_policies=dict(
                type='list',
                elements='dict',
                options=dict(
                    tenant_id=dict(type='str'),
                    object_id=dict(type='str', required=True),
                    application_id=dict(type='str'),
                    # FUTURE: add `choices` support once choices supports lists of values
                    keys=dict(
                        type='list',
                        elements='str',
                        no_log=True,
                        choices=['encrypt', 'decrypt', 'wrapkey', 'unwrapkey', 'sign', 'verify', 'get',
                                 'list', 'create', 'update', 'import', 'delete', 'backup', 'restore', 'recover', 'purge']
                    ),
                    secrets=dict(
                        type='list',
                        elements='str',
                        no_log=True,
                        choices=['get', 'list', 'set', 'delete', 'backup', 'restore', 'recover', 'purge']
                    ),
                    certificates=dict(
                        type='list',
                        elements='str',
                        choices=['get', 'list', 'delete', 'create', 'import', 'update', 'managecontacts',
                                 'getissuers', 'listissuers', 'setissuers', 'deleteissuers', 'manageissuers', 'recover', 'purge', 'backup', 'restore']
                    ),
                    storage=dict(type='list', elements='str')
                )
            ),
            enabled_for_deployment=dict(
                type='bool'
            ),
            enabled_for_disk_encryption=dict(
                type='bool'
            ),
            enabled_for_template_deployment=dict(
                type='bool'
            ),
            enable_soft_delete=dict(
                type='bool',
                default=True
            ),
            enable_rbac_authorization=dict(
                type='bool'
            ),
            soft_delete_retention_in_days=dict(
                type='int'
            ),
            enable_purge_protection=dict(
                type='bool',
                default=False
            ),
            recover_mode=dict(
                type='bool'
            ),
            public_network_access=dict(
                type='str',
                choices=['Disabled', 'Enabled']
            ),
            network_acls=dict(
                type='dict',
                options=dict(
                    bypass=dict(type='str', choices=['AzureServices', 'None'], default='AzureServices'),
                    default_action=dict(type='str', choices=['Allow', 'Deny']),
                    ip_rules=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            value=dict(type='str'),
                        )
                    ),
                    virtual_network_rules=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            id=dict(type='str'),
                            ignore_missing_vnet_service_endpoint=dict(type='bool')
                        )
                    )
                )
            ),
            is_purge_deleted=dict(
                type='bool',
                default=False
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.module_required_if = [['state', 'present', ['vault_tenant']]]

        self.resource_group = None
        self.vault_name = None
        self.hsm_name = None
        self.parameters = dict()
        self.tags = None
        self.identity = None
        self.administrators = None
        self.is_purge_deleted = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.to_do = Actions.NoAction
        self._managed_identity = None

        required_one_of = [('vault_name', 'hsm_name')]
        mutually_exclusive = [('vault_name', 'hsm_name'),
                              ('vault_name', 'identity'),
                              ('vault_name', 'administrators'),
                              ('hsm_name', 'enabled_for_deployment'),
                              ('hsm_name', 'enabled_for_disk_encryption'),
                              ('hsm_name', 'enabled_for_template_deployment'),
                              ('hsm_name', 'access_policies')]

        super(AzureRMVaults, self).__init__(derived_arg_spec=self.module_arg_spec,
                                            supports_check_mode=True,
                                            supports_tags=True,
                                            mutually_exclusive=mutually_exclusive,
                                            required_one_of=required_one_of,
                                            required_if=self.module_required_if)

    @property
    def managed_identity(self):
        if not self._managed_identity:
            self._managed_identity = {"identity": ManagedServiceIdentity,
                                      "user_assigned": UserAssignedIdentity
                                      }
        return self._managed_identity

    def exec_module(self, **kwargs):
        """Main module execution method"""

        # translate Ansible input to SDK-formatted dict in self.parameters
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                if key == "location":
                    self.parameters["location"] = kwargs[key]
                elif key == "vault_tenant":
                    self.parameters.setdefault("properties", {})["tenant_id"] = kwargs[key]
                elif key == "sku" and self.vault_name is not None:
                    self.parameters.setdefault("properties", {})["sku"] = kwargs[key]
                elif key == "sku" and self.hsm_name is not None:
                    self.parameters["sku"] = kwargs[key]
                elif key == "public_network_access":
                    self.parameters.setdefault("properties", {})["public_network_access"] = kwargs[key]
                elif key == "network_acls":
                    self.parameters.setdefault("properties", {})["network_acls"] = kwargs[key]
                elif key == "access_policies":
                    access_policies = kwargs[key]
                    for policy in access_policies:
                        if 'keys' in policy:
                            policy.setdefault("permissions", {})["keys"] = policy["keys"]
                            policy.pop("keys", None)
                        if 'secrets' in policy:
                            policy.setdefault("permissions", {})["secrets"] = policy["secrets"]
                            policy.pop("secrets", None)
                        if 'certificates' in policy:
                            policy.setdefault("permissions", {})["certificates"] = policy["certificates"]
                            policy.pop("certificates", None)
                        if 'storage' in policy:
                            policy.setdefault("permissions", {})["storage"] = policy["storage"]
                            policy.pop("storage", None)
                        if policy.get('tenant_id') is None:
                            # default to key vault's tenant, since that's all that's currently supported anyway
                            policy['tenant_id'] = kwargs['vault_tenant']
                    self.parameters.setdefault("properties", {})["access_policies"] = access_policies
                elif key == "enabled_for_deployment":
                    self.parameters.setdefault("properties", {})["enabled_for_deployment"] = kwargs[key]
                elif key == "enabled_for_disk_encryption":
                    self.parameters.setdefault("properties", {})["enabled_for_disk_encryption"] = kwargs[key]
                elif key == "enabled_for_template_deployment":
                    self.parameters.setdefault("properties", {})["enabled_for_template_deployment"] = kwargs[key]
                elif key == "enable_soft_delete":
                    self.parameters.setdefault("properties", {})["enable_soft_delete"] = kwargs[key]
                elif key == "enable_rbac_authorization":
                    self.parameters.setdefault("properties", {})["enable_rbac_authorization"] = kwargs[key]
                elif key == "enable_purge_protection":
                    self.parameters.setdefault("properties", {})["enable_purge_protection"] = kwargs[key]
                elif key == "soft_delete_retention_in_days":
                    self.parameters.setdefault("properties", {})["soft_delete_retention_in_days"] = kwargs[key]
                elif key == "recover_mode":
                    self.parameters.setdefault("properties", {})["create_mode"] = 'recover' if kwargs[key] else 'default'

        old_response = None
        response = None

        self.mgmt_client = self.get_mgmt_svc_client(KeyVaultManagementClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager,
                                                    api_version="2026-02-01")

        resource_group = self.get_resource_group(self.resource_group)

        if "location" not in self.parameters:
            self.parameters["location"] = resource_group.location

        old_response = self.get_instance()

        curr_identity = old_response.get('identity') if old_response else None
        update_identity, identity_result = self.update_single_managed_identity(curr_identity=curr_identity,
                                                                               new_identity=self.identity)
        if update_identity:
            self.parameters["identity"] = identity_result

        if not old_response:
            self.log("Old instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
                if not self.parameters['properties']['enable_purge_protection']:
                    self.parameters['properties'].pop('enable_purge_protection')
        else:
            self.log("Instance already exists")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            elif self.state == 'present':
                self.log("Need to check if instance has to be deleted or may be updated")
                if not self.parameters['properties']['enable_purge_protection'] and \
                        ('enable_purge_protection' not in old_response['properties'] or
                         not old_response['properties']['enable_purge_protection']):
                    self.parameters['properties'].pop('enable_purge_protection')
                for item in ['enabled_for_deployment', 'enabled_for_disk_encryption', 'enabled_for_template_deployment',
                             'enable_soft_delete', 'enable_purge_protection', 'enable_rbac_authorization']:
                    if item in self.parameters['properties'] and bool(old_response['properties'].get(item)) != bool(self.parameters['properties'][item]):
                        self.to_do = Actions.Update
                    else:
                        self.parameters['properties'][item] = old_response['properties'].get(item)
                if ('location' in self.parameters) and (self.parameters['location'] != old_response['location']):
                    self.to_do = Actions.Update
                elif (('tenant_id' in self.parameters['properties']) and
                        (self.parameters['properties']['tenant_id'] != old_response['properties']['tenant_id'])):
                    self.to_do = Actions.Update
                elif (('soft_delete_retention_in_days' in self.parameters['properties']) and
                        (self.parameters['properties']['soft_delete_retention_in_days'] != old_response['properties'].get('soft_delete_retention_in_days'))):
                    self.to_do = Actions.Update
                elif ('create_mode' in self.parameters['properties']) and (self.parameters['properties']['create_mode'] == 'recover'):
                    self.to_do = Actions.Update
                elif 'access_policies' in self.parameters['properties']:
                    if len(self.parameters['properties']['access_policies']) != len(old_response['properties']['access_policies']):
                        self.to_do = Actions.Update
                    else:
                        # FUTURE: this list isn't really order-dependent- we should be set-ifying the rules list for order-independent comparison
                        for i in range(len(old_response['properties']['access_policies'])):
                            n = self.parameters['properties']['access_policies'][i]
                            o = old_response['properties']['access_policies'][i]
                            if n.get('tenant_id', False) != o.get('tenant_id', False):
                                self.to_do = Actions.Update
                                break
                            if n.get('object_id', None) != o.get('object_id', None):
                                self.to_do = Actions.Update
                                break
                            if n.get('application_id', None) != o.get('application_id', None):
                                self.to_do = Actions.Update
                                break
                            if sorted(n.get('permissions', {}).get('keys', []) or []) != sorted(o.get('permissions', {}).get('keys', []) or []):
                                self.to_do = Actions.Update
                                break
                            if sorted(n.get('permissions', {}).get('secrets', []) or []) != sorted(o.get('permissions', {}).get('secrets', []) or []):
                                self.to_do = Actions.Update
                                break
                            if sorted(n.get('permissions', {}).get('certificates', []) or []) != sorted(o.get('permissions', {}).get('certificates', []) or []):
                                self.to_do = Actions.Update
                                break
                            if sorted(n.get('permissions', {}).get('storage', []) or []) != sorted(o.get('permissions', {}).get('storage', []) or []):
                                self.to_do = Actions.Update
                                break

                if 'public_network_access' in self.parameters['properties'] and \
                        self.parameters['properties']['public_network_access'].capitalize() != old_response['properties'].get('public_network_access'):
                    self.to_do = Actions.Update
                else:
                    self.parameters['properties']['public_network_access'] = old_response['properties'].get('public_network_access')

                if old_response['properties'].get('network_acls') is not None:
                    if old_response['properties']['network_acls'].get('ip_rules') is not None:
                        old_response['properties']['network_acls']['ip_rules'] = list()
                        for item in old_response['properties']['network_acls']['ip_rules']:
                            old_response['properties']['network_acls']['ip_rules'].append(dict(value=item['value'].split('/')[0]))

                if not self.default_compare({}, self.parameters['properties'].get('network_acls'),
                   old_response['properties'].get('network_acls'), '', dict(compare=[])):
                    self.to_do = Actions.Update

                update_tags, newtags = self.update_tags(old_response.get('tags', dict()))

                if self.hsm_name and \
                        self.administrators != old_response["properties"]["initial_admin_object_ids"]:
                    self.to_do = Actions.Update

                if update_tags:
                    self.to_do = Actions.Update
                    self.tags = newtags

                if update_identity:
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the instance")

            self.results['changed'] = True
            if self.check_mode:
                return self.results

            self.parameters["tags"] = self.tags

            if self.vault_name is not None:
                response = self.create_update_keyvault()
            else:
                response = self.create_update_hsm()

            if response is None:
                response = self.get_instance()

            self.log("Creation / Update done")
        elif self.to_do == Actions.Delete:
            self.log("Instance deleted")
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            if self.vault_name is not None:
                self.delete_keyvault()
            else:
                self.hsm_begin_delete()
            # make sure instance is actually deleted, for some Azure resources, instance is hanging around
            # for some time after deletion -- this should be really fixed in Azure
            while self.get_instance():
                time.sleep(20)
        else:
            self.log("Instance unchanged")
            self.results['changed'] = False
            response = old_response

        if response:
            self.results["id"] = response["id"]

        if self.is_purge_deleted:
            purge_response = self.get_deleted(self.parameters["location"])
            if purge_response:
                self.results['changed'] = True
                if not self.check_mode:
                    self.purge_deleted(self.parameters["location"])

        return self.results

    def create_update_keyvault(self):
        '''
        Creates or updates Key Vault with the specified configuration.

        :return: deserialized Key Vault instance state dictionary
        '''
        self.log("Creating / Updating the Key Vault instance {0}".format(self.vault_name))

        try:
            response = self.mgmt_client.vaults.begin_create_or_update(resource_group_name=self.resource_group,
                                                                      vault_name=self.vault_name,
                                                                      parameters=self.parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the Key Vault instance.')
            self.fail("Error creating the Key Vault instance: {0}".format(str(exc)))
        return response and response.as_dict() or None

    def create_update_hsm(self):
        '''
        Creates or updates HSM with the specified configuration.

        :return: deserialized HSM instance state dictionary
        '''
        self.log("Creating / Updating the HSM instance {0}".format(self.hsm_name))

        tenant_id = self.parameters.get('properties', {}).get('tenant_id')
        enable_purge_protection = self.parameters.get('properties', {}).get('enable_purge_protection')
        retention_days = self.parameters.get('properties', {}).get('soft_delete_retention_in_days')
        administrators = self.administrators
        bypass = None
        default_action = None
        public_network_access = None
        properties = ManagedHsmProperties(tenant_id=tenant_id,
                                          enable_purge_protection=enable_purge_protection,
                                          soft_delete_retention_in_days=retention_days,
                                          initial_admin_object_ids=administrators,
                                          network_acls=_create_network_rule_set(bypass, default_action),
                                          public_network_access=public_network_access)
        sku = self.parameters.get('sku')
        parameters = ManagedHsm(location=self.parameters.get('location'),
                                tags=self.parameters.get('tags'),
                                sku=ManagedHsmSku(name=sku.get('name'), family=sku.get('family')),
                                identity=self.parameters.get('identity'),
                                properties=properties)
        try:
            response = self.mgmt_client.managed_hsms.begin_create_or_update(resource_group_name=self.resource_group,
                                                                            name=self.hsm_name,
                                                                            parameters=parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the HSM instance.')
            self.fail("Error creating the HSM instance: {0}".format(str(exc)))
        return response and response.as_dict() or None

    def delete_keyvault(self):
        '''
        Deletes specified Key Vault instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Key Vault instance {0}".format(self.vault_name))
        try:
            response = self.mgmt_client.vaults.delete(resource_group_name=self.resource_group,
                                                      vault_name=self.vault_name)
        except Exception as e:
            self.log('Error attempting to delete the Key Vault instance.')
            self.fail("Error deleting the Key Vault instance: {0}".format(str(e)))

        return True

    def hsm_begin_delete(self):
        '''
        Deletes specified hsm instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the hsm instance {0}".format(self.hsm_name))
        try:
            response = self.mgmt_client.managed_hsms.begin_delete(resource_group_name=self.resource_group,
                                                                  name=self.hsm_name)
            if isinstance(response, LROPoller):
                self.get_poller_result(response)
        except Exception as e:
            self.log('Error attempting to delete the hsm instance.')
            self.fail("Error deleting the hsm instance: {0}".format(str(e)))

        return True

    def get_deleted(self, location):
        '''
        Get deleted vault
        :return: True or False
        '''
        try:
            if self.hsm_name:
                self.log("Get deleted hsm vault instance {0}".format(self.hsm_name))
                self.mgmt_client.managed_hsms.get_deleted(self.hsm_name, location)
            else:
                self.log("Get deleted vault instance {0}".format(self.vault_name))
                self.mgmt_client.vaults.get_deleted(self.vault_name, location)
        except Exception as e:
            self.log('Error attempting to get the deleted vault instance: {0}'.format(str(e)))
            return False
        return True

    def purge_deleted(self, location):
        '''
        Purge vault instance in the specified subscription and resource group.
        '''
        try:
            if self.hsm_name:
                self.log("Purge the deleted hsm vault instance {0}".format(self.hsm_name))
                response = self.mgmt_client.managed_hsms.begin_purge_deleted(self.hsm_name, location)
            else:
                self.log("Purge the deleted vault instance {0}".format(self.vault_name))
                response = self.mgmt_client.vaults.begin_purge_deleted(self.vault_name, location)
        except Exception as e:
            self.log('Error attempting to delete the vault instance.')
            self.fail("Error purge the vault instance: {0}".format(str(e)))

    def get_instance(self):
        '''
        Gets the properties of the specified Key Vault or HSM.

        :return: deserialized Key Vault or HSM instance state dictionary
        '''
        found = False

        if self.vault_name is not None:
            self.log("Checking if the Key Vault instance {0} is present".format(self.vault_name))
            try:
                response = self.mgmt_client.vaults.get(resource_group_name=self.resource_group,
                                                       vault_name=self.vault_name)
                found = True
                self.log("Response : {0}".format(response))
                self.log("Key Vault instance : {0} found".format(response.name))
            except ResourceNotFoundError as e:
                self.log('Did not find the Key Vault instance.')

        if self.hsm_name is not None:
            self.log("Checking if the hsm instance {0} is present".format(self.hsm_name))
            try:
                response = self.mgmt_client.managed_hsms.get(resource_group_name=self.resource_group,
                                                             name=self.hsm_name)
                found = True
                self.log("Response : {0}".format(response))
                self.log("HSM instance : {0} found".format(response.name))
            except ResourceNotFoundError as e:
                self.log('Did not find the hsm instance.')

        if found is True:
            return response.as_dict()

        return False


def _create_network_rule_set(bypass=None, default_action=None):
    return NetworkRuleSet(bypass=bypass or NetworkRuleBypassOptions.azure_services.value,
                          default_action=default_action or NetworkRuleAction.allow.value)


def main():
    """Main execution"""
    AzureRMVaults()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# Copyright: (c) 2020, Cindy Zhao (@cizhao) <cizhao@cisco.com>
# Copyright: (c) 2023, Anvitha Jain (@anvjain) <anvjain@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_tenant
short_description: Manage tenants
description:
- Manage tenants on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
options:
  tenant:
    description:
    - The name of the tenant.
    type: str
    aliases: [ name ]
  display_name:
    description:
    - The name of the tenant to be displayed in the web UI.
    type: str
  description:
    description:
    - The description for this tenant.
    type: str
  users:
    description:
    - A list of associated users for this tenant.
    - Using this property will replace any existing associated users.
    - Admin user is always added to the associated user list irrespective of this parameter being used.
    type: list
    elements: str
  remote_users:
    description:
    - A list of associated remote users for this tenant.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - The name of the associated remote user for this tenant.
        required: true
        type: str
      login_domain:
        description:
        - Domain name of the associated remote user for this tenant.
        required: true
        type: str
  sites:
    description:
    - A list of associated sites for this tenant.
    - Using this property will replace any existing associated sites.
    type: list
    elements: str
  orchestrator_only:
    description:
    - Orchestrator Only C(no) is used to delete the tenant from the MSO and Sites/APIC.
    - C(yes) is used to remove the tenant only from the MSO.
    type: str
    choices: [ 'yes', 'no' ]
    default: 'yes'
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a new tenant
  cisco.mso.mso_tenant:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: north_europe
    display_name: North European Datacenter
    description: This tenant manages the NEDC environment.
    state: present

- name: Remove a tenant from MSO and Site/APIC
  cisco.mso.mso_tenant:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: north_europe
    orchestrator_only: 'no'
    state: absent

- name: Remove a tenant from MSO only
  cisco.mso.mso_tenant:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: north_europe
    orchestrator_only: 'yes'
    state: absent

- name: Query a tenant
  cisco.mso.mso_tenant:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: north_europe
    state: query
  register: query_result

- name: Query all tenants
  cisco.mso.mso_tenant:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    state: query
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, ndo_remote_user_spec
from ansible_collections.cisco.mso.plugins.module_utils.constants import YES_OR_NO_TO_BOOL_STRING_MAP


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        description=dict(type="str"),
        display_name=dict(type="str"),
        tenant=dict(type="str", aliases=["name"]),
        users=dict(type="list", elements="str"),
        remote_users=dict(type="list", elements="dict", options=ndo_remote_user_spec()),
        sites=dict(type="list", elements="str"),
        orchestrator_only=dict(type="str", default="yes", choices=["yes", "no"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant"]],
            ["state", "present", ["tenant"]],
        ],
    )

    description = module.params.get("description")
    display_name = module.params.get("display_name")
    tenant = module.params.get("tenant")
    orchestrator_only = module.params.get("orchestrator_only")
    state = module.params.get("state")
    remote_users = module.params.get("remote_users")

    mso = MSOModule(module)

    tenant_id = None
    path = "tenants"

    # Query for existing object(s)
    if tenant:
        mso.existing = mso.get_obj(path, name=tenant)
        if mso.existing:
            tenant_id = mso.existing.get("id")
            # If we found an existing object, continue with it
            path = "tenants/{id}".format(id=tenant_id)
    else:
        mso.existing = mso.query_objs(path)

    if state == "query":
        pass

    elif state == "absent":
        mso.previous = mso.existing
        if mso.existing:
            if module.check_mode:
                mso.existing = {}
            else:
                path = "{0}?msc-only={1}".format(path, YES_OR_NO_TO_BOOL_STRING_MAP.get(orchestrator_only))
                mso.existing = mso.request(path, method="DELETE")

    elif state == "present":
        mso.previous = mso.existing

        # Convert sites and users
        sites = mso.lookup_sites(module.params.get("sites"))
        users = mso.lookup_users(module.params.get("users"))
        if remote_users is not None:
            users += mso.lookup_remote_users(remote_users)

        payload = dict(
            description=description,
            id=tenant_id,
            name=tenant,
            displayName=display_name,
            siteAssociations=sites,
            userAssociations=users,
        )

        mso.sanitize(payload, collate=True)

        # Ensure displayName is not undefined
        if mso.sent.get("displayName") is None:
            mso.sent["displayName"] = tenant

        if mso.existing:
            if mso.check_changed():
                if module.check_mode:
                    mso.existing = mso.proposed
                else:
                    mso.existing = mso.request(path, method="PUT", data=mso.sent)
        else:
            if module.check_mode:
                mso.existing = mso.proposed
            else:
                mso.existing = mso.request(path, method="POST", data=mso.sent)

    mso.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):
    DOCUMENTATION = r'''
options:
  settings:
    description:
      - Settings assigned to an object.
      - Settings are passed as dict where key is the setting name.
      - Each setting takes the following options value, min, max, writability.
    type: dict
    suboptions:
      value:
        description:
          - The value for the setting.
        type: str
        required: true
      min:
        description:
          - Minimum allowed value for the setting.
        type: str
        required: false
      max:
        description:
          - Maximum allowed value for the setting.
        type: str
        required: false
      writability:
        description:
          - Whether the setting can be changed.
          - Example values you can found L(profiles,https://clickhouse.com/docs/sql-reference/statements/create/settings-profile)
        type: str
        required: false
  profiles:
    description:
      - List of profiles assigned to an object.
    type: list
    elements: str
    required: false
notes:
  - For possible settings, check
    L(session settings,https://clickhouse.com/docs/operations/settings/settings).
'''



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""Generic iterator utilities for chunking data."""

from __future__ import annotations

import typing

if typing.TYPE_CHECKING:
    from typing import Any
    from typing import Iterator

from ansible.module_utils.common.text.converters import to_bytes


def chunks(lst: list, n: int) -> Iterator[list[Any]]:
    """
    Yield successive n-sized chunks from a list.

    :param lst: The list to chunk.
    :param n: The size of each chunk.
    :returns: Iterator yielding list chunks of size n.
    """
    for i in range(0, len(lst), n):
        yield lst[i : i + n]


def chunked_payload(payload: bytes | str, buffer_size: int = 1024) -> Iterator[tuple[bytes, bool]]:
    """
    Yield successive buffer-sized chunks from payload with completion flag.

    Useful for streaming data in fixed-size chunks, with each chunk accompanied
    by a flag indicating whether it's the last chunk.

    :param payload: The data to chunk (bytes or string)
    :param buffer_size: Size of each chunk in bytes (default: 1024)
    :returns: Iterator yielding tuples of (chunk_bytes, is_last_chunk)
    """
    payload_bytes = to_bytes(payload)
    byte_count = len(payload_bytes)
    for i in range(0, byte_count, buffer_size):
        yield payload_bytes[i : i + buffer_size], i + buffer_size >= byte_count



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_saml
short_description: Resource module for organizations _saml
description:
  - Manage operation update of the resource organizations _saml.
  - Updates the SAML SSO enabled settings for an organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  enabled:
    description: Boolean for updating SAML SSO enabled settings.
    type: bool
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  spInitiated:
    description: SP-Initiated SSO settings.
    suboptions:
      idpId:
        description: SAML IdP ID for SP-Initiated SSO Authentication.
        type: str
      subdomain:
        description: Organization's unique SSO identifier.
        type: str
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations updateOrganizationSaml
    description: Complete reference of the updateOrganizationSaml API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-saml
notes:
  - SDK Method used are
    organizations.Organizations.update_organization_saml,
  - Paths used are
    put /organizations/{organizationId}/saml,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.organizations_saml:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    enabled: true
    organizationId: string
    spInitiated:
      idpId: uu3H_bx28Nnd
      subdomain: example_subdomain
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "enabled": true,
      "spInitiated": {
        "subdomain": "string",
        "idpId": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_inventory_orders_claim
short_description: Resource module for organizations _inventory _orders _claim
description:
  - Manage operation create of the resource organizations _inventory _orders _claim.
  - Claim an order by the secure unique order claim number, the order claim id.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  claimId:
    description: The unique order claim id.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  subscriptions:
    description: The individual subscriptions to claim.
    elements: dict
    suboptions:
      description:
        description: Subscription description.
        type: str
      name:
        description: Subscription name.
        type: str
      subscriptionId:
        description: Subscription ID.
        type: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations claimOrganizationInventoryOrders
    description: Complete reference of the claimOrganizationInventoryOrders API.
    link: https://developer.cisco.com/meraki/api-v1/#!claim-organization-inventory-orders
notes:
  - SDK Method used are
    organizations.Organizations.claim_organization_inventory_orders,
  - Paths used are
    post /organizations/{organizationId}/inventory/orders/claim,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_inventory_orders_claim:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    claimId: '12345'
    organizationId: string
    subscriptions:
      - description: Subscription for Wireless Network
        name: My Wireless Subscription
        subscriptionId: SUB1234132
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "claimId": "string",
      "number": "string",
      "serials": [
        "string"
      ],
      "subscriptions": [
        {
          "subscriptionId": "string",
          "name": "string",
          "description": "string",
          "startDate": "string",
          "endDate": "string",
          "isClaimed": true,
          "counts": {
            "seats": {
              "limit": 0
            }
          }
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2026, Ansible community
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: mysql_resource_group

short_description: Add, update, or remove MySQL resource groups

description:
  - Add, update, or remove MySQL resource groups.
  - Resource groups are supported by MySQL 8.0 or later.

version_added: '5.1.0'

options:
  name:
    description:
      - Name of the resource group to manage.
    type: str
    required: true
  state:
    description:
      - If C(present), creates the resource group when it does not exist.
      - If C(present), updates mutable attributes when the resource group already exists.
      - If C(absent), removes the resource group.
    type: str
    choices: [absent, present]
    default: present
  resource_group_type:
    description:
      - Resource group type.
      - Required when creating a new resource group.
      - The value is immutable after creation.
    type: str
    choices: [SYSTEM, USER]
  vcpu_ids:
    description:
      - CPU affinity in native MySQL syntax.
      - Examples include C(0-3) and C(0-3,8,10-12).
      - When omitted for an existing resource group, the current value is left unchanged.
    type: str
  thread_priority:
    description:
      - Thread priority to set for the resource group.
      - Must be between C(-20) and C(0) for C(SYSTEM) resource groups.
      - Must be between C(0) and C(19) for C(USER) resource groups.
      - When omitted for an existing resource group, the current value is left unchanged.
    type: int
  enabled:
    description:
      - Whether the resource group should be enabled.
      - When omitted for an existing resource group, the current value is left unchanged.
    type: bool
  force:
    description:
      - If C(true), use C(FORCE) when dropping a resource group.
      - This only applies when O(state=absent).
    type: bool
    default: false

notes:
  - This module supports MySQL 8.0 or later only.
  - Resource groups are not available on MariaDB.
  - MySQL documents resource groups as unavailable when the thread pool plugin is installed.
  - Thread-priority behavior can depend on server platform and operating-system capabilities.

attributes:
  check_mode:
    support: full
  idempotent:
    support: full

seealso:
  - module: ansible.mysql.mysql_resource_group_info
  - name: MySQL CREATE RESOURCE GROUP reference
    description: Complete reference of the CREATE RESOURCE GROUP command documentation.
    link: https://dev.mysql.com/doc/refman/8.4/en/create-resource-group.html
  - name: MySQL ALTER RESOURCE GROUP reference
    description: Complete reference of the ALTER RESOURCE GROUP command documentation.
    link: https://dev.mysql.com/doc/refman/8.4/en/alter-resource-group.html
  - name: MySQL DROP RESOURCE GROUP reference
    description: Complete reference of the DROP RESOURCE GROUP command documentation.
    link: https://dev.mysql.com/doc/refman/8.4/en/drop-resource-group.html

author:
  - Ron Gershburg (@ronger4)


extends_documentation_fragment:
  - ansible.mysql.mysql
'''

EXAMPLES = r'''
# If you encounter the "Please explicitly state intended protocol" error,
# use the login_unix_socket argument
- name: Create a reporting resource group
  ansible.mysql.mysql_resource_group:
    name: reporting
    resource_group_type: USER
    vcpu_ids: 4-7
    thread_priority: 5
    enabled: true
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Disable a resource group
  ansible.mysql.mysql_resource_group:
    name: reporting
    state: present
    enabled: false

- name: Drop a resource group and reassign running threads
  ansible.mysql.mysql_resource_group:
    name: reporting
    state: absent
    force: true
'''

RETURN = r'''
queries:
  description: List of executed queries.
  returned: when changed
  type: list
  sample: ["CREATE RESOURCE GROUP `reporting` TYPE = USER VCPU = 4-7 THREAD_PRIORITY = 5 ENABLE"]
resource_group:
  description: Normalized representation of the resource group after module execution.
  returned: when O(state=present)
  type: dict
  contains:
    name:
      description: Resource group name.
      type: str
      sample: reporting
    resource_group_type:
      description: Resource group type.
      type: str
      sample: USER
    enabled:
      description: Whether the resource group is enabled.
      type: bool
      sample: true
    vcpu_ids:
      description: Normalized CPU affinity.
      type: str
      sample: 4-7
    thread_priority:
      description: Resource-group thread priority.
      type: int
      sample: 5
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native
from ansible_collections.ansible.mysql.plugins.module_utils.database import mysql_quote_identifier
from ansible_collections.ansible.mysql.plugins.module_utils.mysql import (
    mysql_common_argument_spec,
    mysql_connect,
    mysql_driver,
    mysql_driver_fail_msg,
)
from ansible_collections.ansible.mysql.plugins.module_utils.resource_group import (
    ensure_resource_groups_supported,
    normalize_resource_group_enabled as normalize_enabled,
    normalize_resource_group_row,
    normalize_vcpu_ids,
    validate_resource_group_type,
    validate_thread_priority,
)

DEFAULT_RESOURCE_GROUPS = ('SYS_default', 'USR_default')
RESOURCE_GROUP_QUERY = (
    'SELECT RESOURCE_GROUP_NAME, RESOURCE_GROUP_TYPE, RESOURCE_GROUP_ENABLED, VCPU_IDS, THREAD_PRIORITY '
    'FROM INFORMATION_SCHEMA.RESOURCE_GROUPS WHERE RESOURCE_GROUP_NAME = %s'
)


def normalize_resource_group_inputs(resource_group_type=None, vcpu_ids=None):
    if resource_group_type is not None:
        resource_group_type = validate_resource_group_type(resource_group_type)
    if vcpu_ids is not None:
        vcpu_ids = normalize_vcpu_ids(vcpu_ids)
    return resource_group_type, vcpu_ids


def build_create_query(name, resource_group_type, vcpu_ids=None, thread_priority=None, enabled=None):
    query = [
        'CREATE RESOURCE GROUP %s' % mysql_quote_identifier(name, 'role'),
        'TYPE = %s' % resource_group_type,
    ]

    if vcpu_ids is not None:
        query.append('VCPU = %s' % vcpu_ids)

    if thread_priority is not None:
        query.append('THREAD_PRIORITY = %s' % validate_thread_priority(resource_group_type, thread_priority))

    if enabled is True:
        query.append('ENABLE')
    elif enabled is False:
        query.append('DISABLE')

    return ' '.join(query)


def build_alter_query(name, current, resource_group_type=None, vcpu_ids=None, thread_priority=None, enabled=None):
    if resource_group_type is not None:
        if current['resource_group_type'] != resource_group_type:
            raise ValueError('resource_group_type is immutable after creation')

    changes = []

    if vcpu_ids is not None:
        if current['vcpu_ids'] != vcpu_ids:
            changes.append('VCPU = %s' % vcpu_ids)

    if thread_priority is not None:
        normalized_priority = validate_thread_priority(current['resource_group_type'], thread_priority)
        if current['thread_priority'] != normalized_priority:
            changes.append('THREAD_PRIORITY = %s' % normalized_priority)

    if enabled is not None:
        normalized_enabled = normalize_enabled(enabled)
        if current['enabled'] != normalized_enabled:
            changes.append('ENABLE' if normalized_enabled else 'DISABLE')

    if not changes:
        return None

    return 'ALTER RESOURCE GROUP %s %s' % (mysql_quote_identifier(name, 'role'), ' '.join(changes))


def build_drop_query(name, force=False):
    query = 'DROP RESOURCE GROUP %s' % mysql_quote_identifier(name, 'role')
    if force:
        query += ' FORCE'
    return query


def get_resource_group(cursor, name):
    cursor.execute(RESOURCE_GROUP_QUERY, (name,))
    row = cursor.fetchone()
    if not row:
        return None
    return normalize_resource_group_row(row)


def execute_query(cursor, query):
    cursor.execute(query)


def fail_if_default_resource_group(module, name):
    if name in DEFAULT_RESOURCE_GROUPS:
        module.fail_json(msg='Default resource groups cannot be altered or dropped: %s' % name)


def fail_if_effective_thread_priority_differs(module, requested_thread_priority, current, queries):
    if requested_thread_priority is None:
        return

    if current['thread_priority'] != requested_thread_priority:
        module.fail_json(
            msg=('thread_priority %s could not be applied; server reported %s. '
                 'The server may ignore thread priorities on this platform.'
                 % (requested_thread_priority, current['thread_priority'])),
            changed=True,
            queries=queries,
            resource_group=current,
        )


def main():
    argument_spec = mysql_common_argument_spec()
    argument_spec.update(
        name=dict(type='str', required=True),
        state=dict(type='str', choices=['absent', 'present'], default='present'),
        resource_group_type=dict(type='str', choices=['SYSTEM', 'USER']),
        vcpu_ids=dict(type='str'),
        thread_priority=dict(type='int'),
        enabled=dict(type='bool'),
        force=dict(type='bool', default=False),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    if mysql_driver is None:
        module.fail_json(msg=mysql_driver_fail_msg)

    name = module.params['name']
    state = module.params['state']
    resource_group_type = module.params['resource_group_type']
    vcpu_ids = module.params['vcpu_ids']
    thread_priority = module.params['thread_priority']
    enabled = module.params['enabled']
    force = module.params['force']
    login_user = module.params['login_user']
    login_password = module.params['login_password']
    config_file = module.params['config_file']
    ssl_cert = module.params['client_cert']
    ssl_key = module.params['client_key']
    ssl_ca = module.params['ca_cert']
    connect_timeout = module.params['connect_timeout']
    check_hostname = module.params['check_hostname']

    try:
        cursor, db_conn = mysql_connect(
            module,
            login_user,
            login_password,
            config_file,
            ssl_cert,
            ssl_key,
            ssl_ca,
            connect_timeout=connect_timeout,
            check_hostname=check_hostname,
            cursor_class='DictCursor',
            autocommit=True,
        )
    except Exception as e:
        module.fail_json(msg='unable to connect to database: %s' % to_native(e))

    ensure_resource_groups_supported(module, cursor)

    current = get_resource_group(cursor, name)
    queries = []

    try:
        resource_group_type, vcpu_ids = normalize_resource_group_inputs(resource_group_type, vcpu_ids)
    except ValueError as e:
        module.fail_json(msg=to_native(e))

    if state == 'absent':
        if current is None:
            module.exit_json(changed=False)

        fail_if_default_resource_group(module, name)
        query = build_drop_query(name, force)
        queries.append(query)

        if module.check_mode:
            module.exit_json(changed=True, queries=queries)

        try:
            execute_query(cursor, query)
        except Exception as e:
            module.fail_json(msg=to_native(e))

        module.exit_json(changed=True, queries=queries)

    if current is None:
        if resource_group_type is None:
            module.fail_json(msg='resource_group_type is required when creating a resource group')

        try:
            if thread_priority is not None:
                validate_thread_priority(resource_group_type, thread_priority)
        except ValueError as e:
            module.fail_json(msg=to_native(e))

        query = build_create_query(name, resource_group_type, vcpu_ids, thread_priority, enabled)
        queries.append(query)

        if module.check_mode:
            module.exit_json(
                changed=True,
                queries=queries,
                resource_group={
                    'name': name,
                    'resource_group_type': resource_group_type,
                    'enabled': True if enabled is None else enabled,
                    'vcpu_ids': vcpu_ids,
                    'thread_priority': 0 if thread_priority is None else thread_priority,
                },
            )

        try:
            execute_query(cursor, query)
        except Exception as e:
            module.fail_json(msg=to_native(e))

        current = get_resource_group(cursor, name)
        fail_if_effective_thread_priority_differs(module, thread_priority, current, queries)
        module.exit_json(changed=True, queries=queries, resource_group=current)

    try:
        if thread_priority is not None:
            validate_thread_priority(current['resource_group_type'], thread_priority)
        query = build_alter_query(name, current, resource_group_type, vcpu_ids, thread_priority, enabled)
    except ValueError as e:
        module.fail_json(msg=to_native(e))

    if query is None:
        module.exit_json(changed=False, resource_group=current)

    fail_if_default_resource_group(module, name)
    queries.append(query)

    if module.check_mode:
        module.exit_json(changed=True, queries=queries, resource_group=current)

    try:
        execute_query(cursor, query)
    except Exception as e:
        module.fail_json(msg=to_native(e))

    current = get_resource_group(cursor, name)
    fail_if_effective_thread_priority_differs(module, thread_priority, current, queries)
    module.exit_json(changed=True, queries=queries, resource_group=current)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):
    DOCUMENTATION = r'''
options:
  cluster:
    description:
      - The name of the cluster where the row policy is defined. If not provided, the row policy will be managed on the local server.
    type: str
    default: null
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_summary_top_clients_by_usage_info
short_description: Information module for organizations _summary _top _clients _by
  _usage
description:
  - Get all organizations _summary _top _clients _by _usage.
  - Return metrics for organization's top 10 clients by data usage in mb over given
    time range.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  networkTag:
    description:
      - NetworkTag query parameter. Match result to an exact network tag.
    type: str
  deviceTag:
    description:
      - DeviceTag query parameter. Match result to an exact device tag.
    type: str
  quantity:
    description:
      - Quantity query parameter. Set number of desired results to return. Default
        is 10. Maximum is 50.
    type: int
  ssidName:
    description:
      - SsidName query parameter. Filter results by ssid name.
    type: str
  usageUplink:
    description:
      - UsageUplink query parameter. Filter results by usage uplink.
    type: str
  t0:
    description:
      - T0 query parameter. The beginning of the timespan for the data.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 186 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be greater than or equal to 8 hours and be less than or
        equal to 186 days. The default is 1 day.
    type: float
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationSummaryTopClientsByUsage
    description: Complete reference of the getOrganizationSummaryTopClientsByUsage
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-summary-top-clients-by-usage
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_summary_top_clients_by_usage,
  - Paths used are
    get /organizations/{organizationId}/summary/top/clients/byUsage,
"""

EXAMPLES = r"""
- name: Get all organizations _summary _top _clients _by _usage
  cisco.meraki.organizations_summary_top_clients_by_usage_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkTag: string
    deviceTag: string
    quantity: 0
    ssidName: string
    usageUplink: string
    t0: string
    t1: string
    timespan: 0
    organizationId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "name": "string",
        "mac": "string",
        "id": "string",
        "network": {
          "name": "string",
          "id": "string"
        },
        "usage": {
          "total": 0,
          "upstream": 0,
          "downstream": 0,
          "percentage": 0
        }
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_api_requests_overview_info
short_description: Information module for organizations _api _requests _overview
description:
  - Get all organizations _api _requests _overview.
  - Return an aggregated overview of API requests data.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  t0:
    description:
      - T0 query parameter. The beginning of the timespan for the data. The maximum
        lookback period is 31 days from today.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 31 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be less than or equal to 31 days. The default is 31 days.
    type: float
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationApiRequestsOverview
    description: Complete reference of the getOrganizationApiRequestsOverview API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-api-requests-overview
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_api_requests_overview,
  - Paths used are
    get /organizations/{organizationId}/apiRequests/overview,
"""

EXAMPLES = r"""
- name: Get all organizations _api _requests _overview
  cisco.meraki.organizations_api_requests_overview_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    t0: string
    t1: string
    timespan: 0
    organizationId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "200": 0,
      "201": 0,
      "202": 0,
      "203": 0,
      "204": 0,
      "205": 0,
      "206": 0,
      "207": 0,
      "208": 0,
      "226": 0,
      "300": 0,
      "301": 0,
      "302": 0,
      "303": 0,
      "304": 0,
      "305": 0,
      "306": 0,
      "307": 0,
      "308": 0,
      "400": 0,
      "401": 0,
      "402": 0,
      "403": 0,
      "404": 0,
      "405": 0,
      "406": 0,
      "407": 0,
      "408": 0,
      "409": 0,
      "410": 0,
      "411": 0,
      "412": 0,
      "413": 0,
      "414": 0,
      "415": 0,
      "416": 0,
      "417": 0,
      "421": 0,
      "422": 0,
      "423": 0,
      "424": 0,
      "425": 0,
      "426": 0,
      "428": 0,
      "429": 0,
      "431": 0,
      "451": 0,
      "500": 0
    }
"""



# -*- coding: utf-8 -*-

# Copyright: (c) 2016 Matt Davis, <mdavis@ansible.com>
# Copyright: (c) 2016 Chris Houseknecht, <house@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


class ModuleDocFragment(object):

    # Azure doc fragment
    DOCUMENTATION = r'''

options:
    ad_user:
        description:
            - Active Directory username. Use when authenticating with an Active Directory user rather than service
              principal.
        type: str
    password:
        description:
            - Active Directory user password. Use when authenticating with an Active Directory user rather than service
              principal.
        type: str
    profile:
        description:
            - Security profile found in ~/.azure/credentials file.
        type: str
    subscription_id:
        description:
            - Your Azure subscription Id.
        type: str
    client_id:
        description:
            - Azure client ID. Use when authenticating with a Service Principal or Managed Identity (msi).
            - Can also be set via the C(AZURE_CLIENT_ID) environment variable.
        type: str
    secret:
        description:
            - Azure client secret. Use when authenticating with a Service Principal.
        type: str
    tenant:
        description:
            - Azure tenant ID. Use when authenticating with a Service Principal.
        type: str
    oidc_token:
        description:
            - Direct OpenID Connect (OIDC) token (JWT) used for workload identity authentication.
            - This token is exchanged with Azure AD using OIDC federated identity.
            - Useful in CI systems (for example GitHub Actions, Azure DevOps) where an OIDC token is provided directly by the platform.
            - Can also be set via the C(AZURE_FEDERATED_TOKEN) environment variable.
        type: str
        version_added: '3.17.0'
    oidc_token_file_path:
        description:
            - Path to a file containing an OpenID Connect (OIDC) token (JWT) used for workload identity authentication.
            - The file must contain only raw token.
            - This method is commonly used in platforms that materialize OIDC tokens as files
            - Can also be set via the C(AZURE_FEDERATED_TOKEN_FILE) environment variable.
        type: path
        version_added: '3.17.0'
    cloud_environment:
        description:
            - For cloud environments other than the US public cloud, the environment name (as defined by Azure Python SDK, eg, C(AzureChinaCloud),
              C(AzureUSGovernment)), or a metadata discovery endpoint URL (required for Azure Stack). Can also be set via credential file profile or
              the C(AZURE_CLOUD_ENVIRONMENT) environment variable.
        type: str
        default: AzureCloud
        version_added: '0.0.1'
    adfs_authority_url:
        description:
            - Azure AD authority url. Use when authenticating with Username/password, and has your own ADFS authority.
        type: str
        version_added: '0.0.1'
    cert_validation_mode:
        description:
            - Controls the certificate validation behavior for Azure endpoints. By default, all modules will validate the server certificate, but
              when an HTTPS proxy is in use, or against Azure Stack, it may be necessary to disable this behavior by passing C(ignore). Can also be
              set via credential file profile or the C(AZURE_CERT_VALIDATION) environment variable.
        type: str
        choices: [ ignore, validate ]
        version_added: '0.0.1'
    disable_instance_discovery:
        description:
            - Determines whether or not instance discovery is performed when attempting to authenticate.
              Setting this to true will completely disable both instance discovery and authority validation.
              This functionality is intended for use in scenarios where the metadata endpoint cannot be reached
              such as in private clouds or Azure Stack. The process of instance discovery entails retrieving
              authority metadata from https://login.microsoft.com/ to validate the authority. By setting this
              to **True**, the validation of the authority is disabled. As a result, it is crucial to ensure
              that the configured authority host is valid and trustworthy.
            - Set via credential file profile or the C(AZURE_DISABLE_INSTANCE_DISCOVERY) environment variable.
        type: bool
        default: False
        version_added: '2.3.0'
    auth_source:
        description:
            - Controls the source of the credentials to use for authentication.
            - Can also be set via the C(ANSIBLE_AZURE_AUTH_SOURCE) environment variable.
            - When set to C(auto) (the default) the precedence is module parameters -> C(env) -> C(credential_file) -> C(cli).
            - When set to C(env), the credentials will be read from the environment variables.
            - When set to C(credential_file), it will read the profile from C(~/.azure/credentials).
            - When set to C(cli), the credentials will be sources from the Azure CLI profile. C(subscription_id) or the environment variable
              C(AZURE_SUBSCRIPTION_ID) can be used to identify the subscription ID if more than one is present otherwise the default
              az cli subscription is used.
            - When set to C(msi), the host machine must be an azure resource with an enabled MSI extension. C(subscription_id) or the
              environment variable C(AZURE_SUBSCRIPTION_ID) can be used to identify the subscription ID if the resource is granted
              access to more than one subscription, otherwise the first subscription is chosen.
            - The C(msi) was added in Ansible 2.6.
        type: str
        default: auto
        choices:
        - auto
        - cli
        - credential_file
        - env
        - msi
        version_added: '0.0.1'
    api_profile:
        description:
        - Selects an API profile to use when communicating with Azure services. Default value of C(latest) is appropriate for public clouds;
          future values will allow use with Azure Stack.
        type: str
        default: latest
        version_added: '0.0.1'
    log_path:
        description:
            - Parent argument.
        type: str
    log_mode:
        description:
            - Parent argument.
        type: str
    x509_certificate_path:
        description:
            - Path to the X509 certificate used to create the service principal in PEM format.
            - The certificate must be appended to the private key.
            - Use when authenticating with a Service Principal.
        type: path
        version_added: '1.14.0'
    thumbprint:
        description:
            - The thumbprint of the private key specified in I(x509_certificate_path).
            - Use when authenticating with a Service Principal.
            - Required if I(x509_certificate_path) is defined.
        type: str
        version_added: '1.14.0'
requirements:
    - python >= 2.7
    - The host that executes this module must have the azure.azcollection collection installed via galaxy
    - All python packages listed in collection's requirements.txt must be installed via pip on the host that executes modules from azure.azcollection
    - Full installation instructions may be found https://galaxy.ansible.com/azure/azcollection

notes:
    - For authentication with Azure you can pass parameters, set environment variables, use a profile stored
      in ~/.azure/credentials, or log in before you run your tasks or playbook with C(az login).
    - Authentication is also possible using a service principal or Active Directory user.
    - To authenticate via OIDC, pass client_id, tenant and oidc_token or oidc_token_file_path, or set environment
      variables AZURE_CLIENT_ID, AZURE_TENANT and AZURE_FEDERATED_TOKEN or AZURE_FEDERATED_TOKEN_FILE.
    - To authenticate via service principal, pass subscription_id, client_id, secret and tenant or set environment
      variables AZURE_SUBSCRIPTION_ID, AZURE_CLIENT_ID, AZURE_SECRET and AZURE_TENANT.
    - To authenticate via Active Directory user, pass ad_user and password, or set AZURE_AD_USER and
      AZURE_PASSWORD in the environment.
    - "Alternatively, credentials can be stored in ~/.azure/credentials. This is an ini file containing
      a [default] section and the following keys: subscription_id, client_id, secret and tenant or
      subscription_id, ad_user and password. It is also possible to add additional profiles. Specify the profile
      by passing profile or setting AZURE_PROFILE in the environment."

seealso:
    - name: Sign in with Azure CLI
      link: https://docs.microsoft.com/en-us/cli/azure/authenticate-azure-cli?view=azure-cli-latest
      description: How to authenticate using the C(az login) command.
    '''



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################
"""
The arg spec for the ios_l3_interfaces module
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


class L3_InterfacesArgs(object):
    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "config": {
            "elements": "dict",
            "options": {
                "name": {"type": "str", "required": True},
                "ipv4": {
                    "elements": "dict",
                    "type": "list",
                    "options": {
                        "address": {"type": "str"},
                        "secondary": {"type": "bool"},
                    },
                },
                "ipv6": {
                    "elements": "dict",
                    "type": "list",
                    "options": {"address": {"type": "str"}},
                },
                "carrier_delay": {
                    "type": "dict",
                    "options": {
                        "up": {"type": "int"},
                        "down": {"type": "int"},
                    },
                },
                "dampening": {
                    "type": "dict",
                    "options": {
                        "enabled": {"type": "bool"},
                        "half_life": {"type": "int"},
                        "reuse_threshold": {"type": "int"},
                        "suppress_threshold": {"type": "int"},
                        "max_suppress_time": {"type": "int"},
                        "restart_penalty": {"type": "int"},
                    },
                },
                "load_interval": {"type": "int"},
                "flow_control": {
                    "type": "str",
                    "choices": ["ingress", "egress", "bidirectional"],
                },
                "flow": {
                    "type": "dict",
                    "options": {
                        "ipv4": {
                            "type": "dict",
                            "options": {
                                "monitor": {"type": "str"},
                                "sampler": {"type": "str"},
                                "direction": {
                                    "type": "str",
                                    "choices": ["ingress", "egress", "bidirectional"],
                                },
                            },
                        },
                        "ipv6": {
                            "type": "dict",
                            "options": {
                                "monitor": {"type": "str"},
                                "sampler": {"type": "str"},
                                "direction": {
                                    "type": "str",
                                    "choices": ["ingress", "egress", "bidirectional"],
                                },
                            },
                        },
                    },
                },
            },
            "type": "list",
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "parsed",
                "rendered",
            ],
            "default": "merged",
            "type": "str",
        },
    }



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The ios_lacp class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import to_list

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.utils.utils import dict_to_set


class Lacp(ConfigBase):
    """
    The ios_lacp class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["lacp"]

    def __init__(self, module):
        super(Lacp, self).__init__(module)

    def get_lacp_facts(self, data=None):
        """Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset,
            self.gather_network_resources,
            data=data,
        )
        lacp_facts = facts["ansible_network_resources"].get("lacp")
        if not lacp_facts:
            return dict()

        return lacp_facts

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        commands = list()
        warnings = list()

        if self.state in self.ACTION_STATES:
            existing_lacp_facts = self.get_lacp_facts()
        else:
            existing_lacp_facts = dict()

        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_lacp_facts))
        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self._connection.edit_config(commands)
            result["changed"] = True
        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_lacp_facts = self.get_lacp_facts()
        elif self.state == "rendered":
            result["rendered"] = commands
        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed",
                )
            result["parsed"] = self.get_lacp_facts(data=running_config)
        else:
            changed_lacp_facts = dict()

        if self.state in self.ACTION_STATES:
            result["before"] = existing_lacp_facts
            if result["changed"]:
                result["after"] = changed_lacp_facts
        elif self.state == "gathered":
            result["gathered"] = changed_lacp_facts
        result["warnings"] = warnings

        return result

    def set_config(self, existing_lacp_facts):
        """Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        want = self._module.params["config"]
        have = existing_lacp_facts
        resp = self.set_state(want, have)

        return to_list(resp)

    def set_state(self, want, have):
        """Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        if self.state in ("merged", "replaced", "overridden", "rendered") and not want:
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(self.state),
            )

        if self.state == "deleted":
            commands = self._state_deleted(want, have)
        elif self.state in ("merged", "rendered"):
            commands = self._state_merged(want, have)
        elif self.state in ["replaced", "overridden"]:
            commands = self._state_replaced(want, have)

        return commands

    def _state_replaced(self, want, have):
        """The command generator when state is replaced

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []

        commands.extend(self._set_config(want, have))

        return commands

    def _state_merged(self, want, have):
        """The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        commands = []

        commands.extend(self._set_config(want, have))

        return commands

    def _state_deleted(self, want, have):
        """The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []

        if want:
            commands.extend(self._clear_config(have))
        else:
            commands.extend(self._clear_config(have))

        return commands

    def _remove_command_from_config_list(self, cmd, commands):
        commands.append("no %s" % cmd)
        return commands

    def _add_command_to_config_list(self, cmd, commands):
        if cmd not in commands:
            commands.append(cmd)

    def _set_config(self, want, have):
        # Set the interface config based on the want and have config
        commands = []

        want_dict = dict_to_set(want)
        have_dict = dict_to_set(have)
        diff = want_dict - have_dict

        if diff:
            cmd = "lacp system-priority {0}".format(want.get("system").get("priority"))
            self._add_command_to_config_list(cmd, commands)

        return commands

    def _clear_config(self, have):
        # Delete the interface config based on the want and have config
        commands = []

        if have.get("system").get("priority") and have.get("system").get("priority") != 32768:
            cmd = "lacp system-priority"
            self._remove_command_from_config_list(cmd, commands)

        return commands



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""Generic S3 utilities for AWS plugins."""

from __future__ import annotations

import typing

if typing.TYPE_CHECKING:
    from typing import Dict
    from typing import Optional
    from typing import Tuple


def escape_path(path: str) -> str:
    """
    Converts a file path to S3-compatible format by replacing backslashes with forward slashes.

    S3 uses forward slashes as path separators regardless of the operating system.
    Windows paths with backslashes must be converted to ensure proper S3 key formatting.

    :param path: The file path to convert
    :returns: The path with forward slashes
    """
    return path.replace("\\", "/")


def generate_encryption_settings(
    bucket_sse_mode: Optional[str], bucket_sse_kms_key_id: Optional[str]
) -> Tuple[Dict, Dict]:
    """
    Generate S3 server-side encryption settings for boto3 and HTTP headers.

    :param bucket_sse_mode: Server-side encryption mode (e.g. 'AES256', 'aws:kms')
    :param bucket_sse_kms_key_id: KMS key ID for aws:kms encryption (optional)
    :returns: Tuple of (boto3_args, http_headers) containing encryption settings
    """
    args = {}
    headers = {}
    if not bucket_sse_mode:
        return args, headers

    args["ServerSideEncryption"] = bucket_sse_mode
    headers["x-amz-server-side-encryption"] = bucket_sse_mode
    if bucket_sse_mode == "aws:kms" and bucket_sse_kms_key_id:
        args["SSEKMSKeyId"] = bucket_sse_kms_key_id
        headers["x-amz-server-side-encryption-aws-kms-key-id"] = bucket_sse_kms_key_id
    return args, headers



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017, 2018 Michael De La Rue
# Copyright (c) 2017, 2018 Will Thames
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: rds_instance_info
version_added: 5.0.0
short_description: obtain information about one or more RDS instances
description:
  - Obtain information about one or more RDS instances.
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  db_instance_identifier:
    description:
      - The RDS instance's unique identifier.
    required: false
    aliases:
      - id
    type: str
  filters:
    description:
      - A filter that specifies one or more DB instances to describe.
        See U(https://docs.aws.amazon.com/AmazonRDS/latest/APIReference/API_DescribeDBInstances.html)
    type: dict
author:
  - "Will Thames (@willthames)"
  - "Michael De La Rue (@mikedlr)"
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Get information about an instance
  amazon.aws.rds_instance_info:
    db_instance_identifier: new-database
  register: new_database_info

- name: Get all RDS instances
  amazon.aws.rds_instance_info:
"""

RETURN = r"""
instances:
  description: List of RDS instances
  returned: always
  type: complex
  contains:
    allocated_storage:
      description: Gigabytes of storage allocated to the database.
      returned: always
      type: int
      sample: 10
    auto_minor_version_upgrade:
      description: Whether minor version upgrades happen automatically.
      returned: always
      type: bool
      sample: true
    availability_zone:
      description: Availability Zone in which the database resides.
      returned: always
      type: str
      sample: us-west-2b
    backup_retention_period:
      description: Days for which backups are retained.
      returned: always
      type: int
      sample: 7
    ca_certificate_identifier:
      description: ID for the CA certificate.
      returned: always
      type: str
      sample: rds-ca-2015
    copy_tags_to_snapshot:
      description: Whether DB tags should be copied to the snapshot.
      returned: always
      type: bool
      sample: false
    db_instance_arn:
      description: ARN of the database instance.
      returned: always
      type: str
      sample: arn:aws:rds:us-west-2:123456789012:db:helloworld-rds
    db_instance_class:
      description: Instance class of the database instance.
      returned: always
      type: str
      sample: db.t3.small
    db_instance_identifier:
      description: Database instance identifier.
      returned: always
      type: str
      sample: helloworld-rds
    db_instance_port:
      description: Port used by the database instance.
      returned: always
      type: int
      sample: 0
    db_instance_status:
      description: Status of the database instance.
      returned: always
      type: str
      sample: available
    db_name:
      description: Name of the database.
      returned: always
      type: str
      sample: management
    db_parameter_groups:
      description: List of database parameter groups.
      returned: always
      type: complex
      contains:
        db_parameter_group_name:
          description: Name of the database parameter group.
          returned: always
          type: str
          sample: psql-pg-helloworld
        parameter_apply_status:
          description: Whether the parameter group has been applied.
          returned: always
          type: str
          sample: in-sync
    db_security_groups:
      description: List of security groups used by the database instance.
      returned: always
      type: list
      sample: []
    db_subnet_group:
      description: List of subnet groups.
      returned: always
      type: complex
      contains:
        db_subnet_group_description:
          description: Description of the DB subnet group.
          returned: always
          type: str
          sample: My database subnet group
        db_subnet_group_name:
          description: Name of the database subnet group.
          returned: always
          type: str
          sample: my-subnet-group
        subnet_group_status:
          description: Subnet group status.
          returned: always
          type: str
          sample: Complete
        subnets:
          description: List of subnets in the subnet group.
          returned: always
          type: complex
          contains:
            subnet_availability_zone:
              description: Availability zone of the subnet.
              returned: always
              type: complex
              contains:
                name:
                  description: Name of the availability zone.
                  returned: always
                  type: str
                  sample: us-west-2c
            subnet_identifier:
              description: Subnet ID.
              returned: always
              type: str
              sample: subnet-abcd1234
            subnet_status:
              description: Subnet status.
              returned: always
              type: str
              sample: Active
        vpc_id:
          description: VPC id of the subnet group.
          returned: always
          type: str
          sample: vpc-abcd1234
    dbi_resource_id:
      description: AWS Region-unique, immutable identifier for the DB instance.
      returned: always
      type: str
      sample: db-AAAAAAAAAAAAAAAAAAAAAAAAAA
    deletion_protection:
      description: C(True) if the DB instance has deletion protection enabled, C(False) if not.
      returned: always
      type: bool
      sample: False
      version_added: 3.3.0
      version_added_collection: community.aws
    domain_memberships:
      description: List of domain memberships.
      returned: always
      type: list
      sample: []
    endpoint:
      description: Database endpoint
      returned: always
      type: complex
      contains:
        address:
          description: Database endpoint address.
          returned: always
          type: str
          sample: helloworld-rds.ctrqpe3so1sf.us-west-2.rds.amazonaws.com
        hosted_zone_id:
          description: Route 53 hosted zone ID.
          returned: always
          type: str
          sample: Z1PABCD0000000
        port:
          description: Database endpoint port.
          returned: always
          type: int
          sample: 5432
    engine:
      description: Database engine.
      returned: always
      type: str
      sample: postgres
    engine_version:
      description: Database engine version.
      returned: always
      type: str
      sample: 9.5.10
    iam_database_authentication_enabled:
      description: Whether database authentication through IAM is enabled.
      returned: always
      type: bool
      sample: false
    instance_create_time:
      description: Date and time the instance was created.
      returned: always
      type: str
      sample: '2017-10-10T04:00:07.434000+00:00'
    iops:
      description: The Provisioned IOPS value for the DB instance.
      returned: always
      type: int
      sample: 1000
    kms_key_id:
      description: KMS Key ID.
      returned: always
      type: str
      sample: arn:aws:kms:us-west-2:123456789012:key/abcd1234-0000-abcd-1111-0123456789ab
    latest_restorable_time:
      description: Latest time to which a database can be restored with point-in-time restore.
      returned: always
      type: str
      sample: '2018-05-17T00:03:56+00:00'
    license_model:
      description: License model.
      returned: always
      type: str
      sample: postgresql-license
    master_username:
      description: Database master username.
      returned: always
      type: str
      sample: dbadmin
    monitoring_interval:
      description: Interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.
      returned: always
      type: int
      sample: 0
    multi_az:
      description: Whether Multi-AZ is on.
      returned: always
      type: bool
      sample: false
    option_group_memberships:
      description: List of option groups.
      returned: always
      type: complex
      contains:
        option_group_name:
          description: Option group name.
          returned: always
          type: str
          sample: default:postgres-9-5
        status:
          description: Status of option group.
          returned: always
          type: str
          sample: in-sync
    pending_modified_values:
      description: Modified values pending application.
      returned: always
      type: complex
      contains: {}
    performance_insights_enabled:
      description: Whether performance insights are enabled.
      returned: always
      type: bool
      sample: false
    preferred_backup_window:
      description: Preferred backup window.
      returned: always
      type: str
      sample: 04:00-05:00
    preferred_maintenance_window:
      description: Preferred maintenance window.
      returned: always
      type: str
      sample: mon:05:00-mon:05:30
    publicly_accessible:
      description: Whether the DB is publicly accessible.
      returned: always
      type: bool
      sample: false
    read_replica_db_instance_identifiers:
      description: List of database instance read replicas.
      returned: always
      type: list
      sample: []
    storage_encrypted:
      description: Whether the storage is encrypted.
      returned: always
      type: bool
      sample: true
    storage_type:
      description: Storage type of the Database instance.
      returned: always
      type: str
      sample: gp2
    tags:
      description: Tags used by the database instance.
      returned: always
      type: complex
      contains: {}
    vpc_security_groups:
      description: List of VPC security groups.
      returned: always
      type: complex
      contains:
        status:
          description: Status of the VPC security group.
          returned: always
          type: str
          sample: active
        vpc_security_group_id:
          description: VPC Security Group ID.
          returned: always
          type: str
          sample: sg-abcd1234
"""

from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Union

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.rds import AnsibleRDSError
from ansible_collections.amazon.aws.plugins.module_utils.rds import describe_db_instances
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def instance_info(
    client, module: AnsibleAWSModule, instance_name: Optional[str], filters: Optional[Dict[str, Union[str, List]]]
) -> List[Dict[str, Any]]:
    """
    Returns attributes of db instance(s), with instances optionally filtered by provided name and additional filters.

        Parameters:
            client: boto3 rds client
            module: AnsibleAWSModule
            instance_name (str, optional): Unique identifier of db instance to describe
            filters (dict, optional): Additional boto3-supported filters specifying db instance(s) to describe

        Returns:
            instances (list): List of instance attribute dicts converted from CamelCase to snake_case format
    """
    params = {}
    if instance_name:
        params["DBInstanceIdentifier"] = instance_name
    if filters:
        params["Filters"] = ansible_dict_to_boto3_filter_list(filters)

    results = describe_db_instances(client, **params)
    for instance in results:
        instance["Tags"] = boto3_tag_list_to_ansible_dict(instance.pop("TagList"))

    return [camel_dict_to_snake_dict(instance, ignore_list=["Tags"]) for instance in results]


def main():
    argument_spec = dict(
        db_instance_identifier=dict(aliases=["id"]),
        filters=dict(type="dict"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    client = module.client("rds")

    instance_name = module.params.get("db_instance_identifier")
    filters = module.params.get("filters")

    try:
        module.exit_json(changed=False, instances=instance_info(client, module, instance_name, filters))
    except AnsibleRDSError as e:
        module.fail_json_aws(e)


if __name__ == "__main__":
    main()



# Copyright (c) 2021 Felix Fontein
# Copyright (c) 2020 Markus Bergholz <markuman+spambelongstogoogle@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: hetzner_dns_records

short_description: Create inventory from Hetzner DNS records

version_added: 2.0.0

description:
  - This plugin allows to create an inventory from Hetzner DNS records.
  - 'For Ansible to be able to identify a YAML file as an inventory for this plugin, the inventory file must contain C(plugin:
    community.dns.hetzner_dns_records) and its filename must end with C(hetzner_dns.yaml) or C(hetzner_dns.yml).'
options:
  plugin:
    description: The name of this plugin. Should always be set to V(community.dns.hetzner_dns_records) for this plugin to
      recognize it as its own.
    required: true
    choices:
      - community.dns.hetzner_dns_records
    type: str

  filters:
    version_added: 3.0.0

extends_documentation_fragment:
  - community.dns._hetzner
  - community.dns._hetzner.plugin
  - community.dns._hetzner.record_type_choices_records_inventory
  - community.dns._hetzner.record_type_seealso
  - community.dns._hetzner.zone_id_type
  - community.dns._inventory_records
  - community.dns._options.record_transformation
  - community.library_inventory_filtering_v1.inventory_filter

notes:
  - The provider-specific O(hetzner_token) option can be templated.
author:
  - Markus Bergholz (@markuman) <markuman+spambelongstogoogle@gmail.com>
  - Felix Fontein (@felixfontein)

seealso:
  - module: community.dns.hetzner_dns_record_set_info
  - module: community.dns.hetzner_dns_record_info
"""

EXAMPLES = r"""
# filename must end with hetzner_dns.yaml or hetzner_dns.yml

plugin: community.dns.hetzner_dns_records
zone_name: domain.de
simple_filters:
  type:
    - TXT
filters:
  - include: >-
      not ansible_host.startswith('v=')
  - exclude: true
txt_transformation: unquoted

# You can also configure the token by putting secret value into this file,
# but this is discouraged. Use a lookup like below, or leave it away and
# set it with the HETZNER_DNS_TOKEN environment variable.
hetzner_token: >-
  {{ (lookup('community.sops.sops', 'keys/hetzner.sops.yml') | from_yaml).hetzner_dns_token }}
"""

import typing as t

from ansible_collections.community.dns.plugins.module_utils._hetzner.api import (
    create_hetzner_api,
    create_hetzner_provider_information,
)
from ansible_collections.community.dns.plugins.module_utils._http import OpenURLHelper
from ansible_collections.community.dns.plugins.plugin_utils._inventory.records import (
    RecordsInventoryModule,
)
from ansible_collections.community.dns.plugins.plugin_utils._templated_options import (
    TemplatedOptionProvider,
)

if t.TYPE_CHECKING:  # pragma: no cover
    from ..module_utils._provider import ProviderInformation
    from ..module_utils._zone_record_api import ZoneRecordAPI
    from ..module_utils._zone_record_set_api import ZoneRecordSetAPI


class InventoryModule(RecordsInventoryModule):
    NAME = "community.dns.hetzner_dns_records"
    VALID_ENDINGS = ("hetzner_dns.yaml", "hetzner_dns.yml")

    def setup_api(self) -> tuple[ProviderInformation, ZoneRecordAPI | ZoneRecordSetAPI]:
        option_provider = TemplatedOptionProvider(self, self.templar)
        provider_information = create_hetzner_provider_information(
            option_provider=option_provider
        )
        return provider_information, create_hetzner_api(
            option_provider, OpenURLHelper()
        )
